@@ -32,8 +32,11 @@
 #include <src/udiskslogging.h>
 #include <src/udisksmodulemanager.h>
 
+#include "udisks-iscsi-generated.h"
+#include "udisksiscsistate.h"
 #include "udisksiscsiutil.h"
 #include "udiskslinuxmanageriscsiinitiator.h"
+#include "udisksiscsidbusutil.h"
 
 /**
  * SECTION:udiskslinuxmanageriscsiinitiator
@@ -51,10 +54,11 @@
  * The #UDisksLinuxManagerISCSIInitiator structure contains only private data
  * and should only be accessed using the provided API.
  */
-struct _UDisksLinuxManagerISCSIInitiator {
+struct _UDisksLinuxManagerISCSIInitiator{
   UDisksManagerISCSIInitiatorSkeleton parent_instance;
 
-  UDisksLinuxModuleISCSI *module;
+  UDisksDaemon *daemon;
+  UDisksISCSIState *state;
   GMutex initiator_config_mutex;  /* We use separate mutex for configuration
                                      file because libiscsi doesn't provide us
                                      any API for this. */
@@ -67,7 +71,7 @@ struct _UDisksLinuxManagerISCSIInitiatorClass {
 enum
 {
   PROP_0,
-  PROP_MODULE
+  PROP_DAEMON
 };
 
 static void udisks_linux_manager_iscsi_initiator_iface_init (UDisksManagerISCSIInitiatorIface *iface);
@@ -77,8 +81,8 @@ G_DEFINE_TYPE_WITH_CODE (UDisksLinuxManagerISCSIInitiator, udisks_linux_manager_
                          G_IMPLEMENT_INTERFACE (UDISKS_TYPE_MANAGER_ISCSI_INITIATOR,
                                                 udisks_linux_manager_iscsi_initiator_iface_init));
 
-#define INITIATOR_FILENAME "/etc/iscsi/initiatorname.iscsi"
-#define INITIATOR_NAME_KEY "InitiatorName"
+const gchar *initiator_filename = "/etc/iscsi/initiatorname.iscsi";
+const gchar *initiator_name_prefix = "InitiatorName=";
 
 /* ---------------------------------------------------------------------------------------------------- */
 
@@ -90,8 +94,8 @@ udisks_linux_manager_iscsi_initiator_get_property (GObject *object, guint proper
 
   switch (property_id)
     {
-    case PROP_MODULE:
-      g_value_set_object (value, udisks_linux_manager_iscsi_initiator_get_module (manager));
+    case PROP_DAEMON:
+      g_value_set_object (value, udisks_linux_manager_iscsi_initiator_get_daemon (manager));
       break;
 
     default:
@@ -108,9 +112,10 @@ udisks_linux_manager_iscsi_initiator_set_property (GObject *object, guint proper
 
   switch (property_id)
     {
-    case PROP_MODULE:
-      g_assert (manager->module == NULL);
-      manager->module = g_value_dup_object (value);
+    case PROP_DAEMON:
+      g_assert (manager->daemon == NULL);
+      /* We don't take a reference to the daemon */
+      manager->daemon = g_value_get_object (value);
       break;
 
     default:
@@ -119,6 +124,26 @@ udisks_linux_manager_iscsi_initiator_set_property (GObject *object, guint proper
     }
 }
 
+static void
+udisks_linux_manager_iscsi_initiator_constructed (GObject *object)
+{
+  UDisksLinuxManagerISCSIInitiator *manager;
+  UDisksModuleManager *module_manager;
+
+  /* Store state pointer for later usage; libiscsi_context. */
+  manager = UDISKS_LINUX_MANAGER_ISCSI_INITIATOR (object);
+  module_manager = udisks_daemon_get_module_manager (manager->daemon);
+  manager->state = udisks_module_manager_get_module_state_pointer (module_manager,
+                                                                   ISCSI_MODULE_NAME);
+}
+
+static void
+udisks_linux_manager_iscsi_initiator_dispose (GObject *object)
+{
+  if (G_OBJECT_CLASS (udisks_linux_manager_iscsi_initiator_parent_class))
+    G_OBJECT_CLASS (udisks_linux_manager_iscsi_initiator_parent_class)->dispose (object);
+}
+
 static void
 udisks_linux_manager_iscsi_initiator_finalize (GObject *object)
 {
@@ -133,19 +158,20 @@ udisks_linux_manager_iscsi_initiator_class_init (UDisksLinuxManagerISCSIInitiato
 
   gobject_class->get_property = udisks_linux_manager_iscsi_initiator_get_property;
   gobject_class->set_property = udisks_linux_manager_iscsi_initiator_set_property;
+  gobject_class->constructed = udisks_linux_manager_iscsi_initiator_constructed;
+  gobject_class->dispose = udisks_linux_manager_iscsi_initiator_dispose;
   gobject_class->finalize = udisks_linux_manager_iscsi_initiator_finalize;
 
-  /**
-   * UDisksLinuxManager:module:
+  /** UDisksLinuxManager:daemon
    *
-   * The #UDisksLinuxModuleISCSI for the object.
+   * The #UDisksDaemon for the object.
    */
   g_object_class_install_property (gobject_class,
-                                   PROP_MODULE,
-                                   g_param_spec_object ("module",
-                                                        "Module",
-                                                        "The module for the object",
-                                                        UDISKS_TYPE_LINUX_MODULE_ISCSI,
+                                   PROP_DAEMON,
+                                   g_param_spec_object ("daemon",
+                                                        "Daemon",
+                                                        "The daemon for the object",
+                                                        UDISKS_TYPE_DAEMON,
                                                         G_PARAM_READABLE |
                                                         G_PARAM_WRITABLE |
                                                         G_PARAM_CONSTRUCT_ONLY |
@@ -155,6 +181,8 @@ udisks_linux_manager_iscsi_initiator_class_init (UDisksLinuxManagerISCSIInitiato
 static void
 udisks_linux_manager_iscsi_initiator_init (UDisksLinuxManagerISCSIInitiator *manager)
 {
+  manager->daemon = NULL;
+
   g_dbus_interface_skeleton_set_flags (G_DBUS_INTERFACE_SKELETON (manager),
                                        G_DBUS_INTERFACE_SKELETON_FLAGS_HANDLE_METHOD_INVOCATIONS_IN_THREAD);
 #ifdef HAVE_LIBISCSI_GET_SESSION_INFOS
@@ -165,34 +193,65 @@ udisks_linux_manager_iscsi_initiator_init (UDisksLinuxManagerISCSIInitiator *man
 
 /**
  * udisks_linux_manager_iscsi_initiator_new:
- * @module: A #UDisksLinuxModuleISCSI.
+ * @daemon: A #UDisksDaemon.
  *
  * Creates a new #UDisksLinuxManagerISCSIInitiator instance.
  *
  * Returns: A new #UDisksLinuxManagerISCSIInitiator. Free with g_object_unref().
  */
 UDisksLinuxManagerISCSIInitiator *
-udisks_linux_manager_iscsi_initiator_new (UDisksLinuxModuleISCSI *module)
+udisks_linux_manager_iscsi_initiator_new (UDisksDaemon *daemon)
 {
-  g_return_val_if_fail (UDISKS_IS_LINUX_MODULE_ISCSI (module), NULL);
+  g_return_val_if_fail (UDISKS_IS_DAEMON (daemon), NULL);
   return UDISKS_LINUX_MANAGER_ISCSI_INITIATOR (g_object_new (UDISKS_TYPE_LINUX_MANAGER_ISCSI_INITIATOR,
-                                                             "module", module,
+                                                             "daemon", daemon,
                                                              NULL));
 }
 
 /**
- * udisks_linux_manager_iscsi_initiator_get_module:
+ * udisks_linux_manager_iscsi_initiator_get_daemon:
  * @manager: A #UDisksLinuxManagerISCSIInitiator.
  *
- * Gets the module used by @manager.
+ * Gets the daemon used by @manager.
  *
- * Returns: A #UDisksLinuxModuleISCSI. Do not free, the object is owned by @manager.
+ * Returns: A #UDisksDaemon. Do not free, the object is owned by @manager.
  */
-UDisksLinuxModuleISCSI *
-udisks_linux_manager_iscsi_initiator_get_module (UDisksLinuxManagerISCSIInitiator *manager)
+UDisksDaemon *
+udisks_linux_manager_iscsi_initiator_get_daemon (UDisksLinuxManagerISCSIInitiator *manager)
 {
   g_return_val_if_fail (UDISKS_IS_LINUX_MANAGER_ISCSI_INITIATOR (manager), NULL);
-  return manager->module;
+  return manager->daemon;
+}
+
+/**
+ * udisks_linux_manager_iscsi_initiator_get_state:
+ * @manager: A #UDisksLinuxManagerISCSIInitiator.
+ *
+ * Gets the state pointer for iSCSI module.
+ *
+ * Returns: A #UDisksISCSIState. Do not free, the structure is owned by
+ * #UDisksModuleManager.
+ */
+static UDisksISCSIState *
+udisks_linux_manager_iscsi_initiator_get_state (UDisksLinuxManagerISCSIInitiator *manager)
+{
+  g_return_val_if_fail (UDISKS_IS_LINUX_MANAGER_ISCSI_INITIATOR (manager), NULL);
+  return manager->state;
+}
+
+static struct libiscsi_context *
+udisks_linux_manager_iscsi_initiator_get_iscsi_context (UDisksLinuxManagerISCSIInitiator *manager)
+{
+  UDisksModuleManager *module_manager;
+  UDisksISCSIState *state;
+
+  g_return_val_if_fail (UDISKS_IS_LINUX_MANAGER_ISCSI_INITIATOR (manager), NULL);
+
+  module_manager = udisks_daemon_get_module_manager (manager->daemon);
+  state = udisks_module_manager_get_module_state_pointer (module_manager,
+                                                          ISCSI_MODULE_NAME);
+
+  return udisks_iscsi_state_get_libiscsi_context (state);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -223,68 +282,73 @@ handle_get_firmware_initiator_name (UDisksManagerISCSIInitiator *object,
   return TRUE;
 }
 
-
-#define FAKE_GROUP_NAME "general"       /* NOTE: shows up in some error messages */
-
-static gchar *
-_get_initiator_name (GError **error)
-{
-  gchar *initiator_name;
-  gchar *contents = NULL;
-  gchar *contents_group;
-  GKeyFile *key_file;
-
-  if (! g_file_get_contents (INITIATOR_FILENAME, &contents, NULL, error))
-    {
-      g_prefix_error (error, "Error reading iSCSI initiator name from '%s': ", INITIATOR_FILENAME);
-      return NULL;
-    }
-
-  /* key-value pairs have to belong to some group, let's create a fake one just to satisfy the parser needs */
-  contents_group = g_strconcat ("[" FAKE_GROUP_NAME "]\n", contents, NULL);
-  g_free (contents);
-
-  key_file = g_key_file_new ();
-  if (! g_key_file_load_from_data (key_file, contents_group, -1, G_KEY_FILE_NONE, error))
-    {
-      g_prefix_error (error, "Error reading iSCSI initiator name from '%s': ", INITIATOR_FILENAME);
-      g_key_file_free (key_file);
-      g_free (contents_group);
-      return NULL;
-    }
-
-  initiator_name = g_key_file_get_string (key_file, FAKE_GROUP_NAME, INITIATOR_NAME_KEY, error);
-  if (initiator_name == NULL)
-    g_prefix_error (error, "Error reading iSCSI initiator name from '%s': ", INITIATOR_FILENAME);
-
-  g_key_file_free (key_file);
-  g_free (contents_group);
-
-  /* trim the whitespace at the end of the string */
-  if (initiator_name != NULL)
-    initiator_name = g_strchomp (initiator_name);
-
-  return initiator_name;
-}
-
 static gboolean
 handle_get_initiator_name (UDisksManagerISCSIInitiator *object,
                            GDBusMethodInvocation       *invocation)
 {
   UDisksLinuxManagerISCSIInitiator *manager = UDISKS_LINUX_MANAGER_ISCSI_INITIATOR (object);
-  gchar *initiator_name = NULL;
-  GError *error = NULL;
+
+  int initiator_name_fd = -1;
+  int nbytes;
+  gchar buf[64]; /* Do we need more? */
+  gchar *initiator_name;
+  size_t len;
+  GString *content = NULL;
 
   /* Enter a critical section. */
   g_mutex_lock (&manager->initiator_config_mutex);
 
-  initiator_name = _get_initiator_name(&error);
-  if (!initiator_name)
+  initiator_name_fd = open (initiator_filename, O_RDONLY);
+  if (initiator_name_fd == -1)
     {
-      g_dbus_method_invocation_take_error (invocation, error);
+      g_dbus_method_invocation_return_error (invocation,
+                                             UDISKS_ERROR,
+                                             UDISKS_ERROR_FAILED,
+                                             N_("Error opening %s while getting iSCSI initiator name: %s"),
+                                             initiator_filename,
+                                             strerror (errno));
+
       goto out;
     }
 
+  /* Read the initiator name. */
+  content = g_string_new (NULL);
+  while ((nbytes = read (initiator_name_fd, buf, sizeof (buf) - 1)) > 0)
+    {
+      buf[nbytes] = '\0';
+      content = g_string_append (content, buf);
+    }
+
+  if (nbytes < 0)
+    {
+      g_dbus_method_invocation_return_error (invocation,
+                                             UDISKS_ERROR,
+                                             UDISKS_ERROR_FAILED,
+                                             N_("Error reading %s while getting iSCSI initiator name: %s"),
+                                             initiator_filename,
+                                             strerror (errno));
+
+      goto out;
+    }
+
+  /* We don't want to create a scanner for this iscsi initiator name grammar.
+   * So for simplicity, we just search for "InitiatorName=" prefix and if
+   * found, we just shift the name pointer.  The string may contain whitespace
+   * at the end; it's removed as well. */
+  len = strlen (initiator_name_prefix);
+  initiator_name = content->str;
+  if (strncmp (content->str, initiator_name_prefix, len) == 0)
+    {
+      /* Shift the pointer by prefix length further. */
+      initiator_name = &content->str[len];
+    }
+  /* Trim the whitespace at the end of the string. */
+  while (g_ascii_isspace (content->str[content->len - 1]))
+    {
+      content->str[content->len - 1] = '\0';
+      content->len--;
+    }
+
   /* Return the initiator name */
   udisks_manager_iscsi_initiator_complete_get_initiator_name (object,
                                                               invocation,
@@ -295,41 +359,9 @@ out:
   g_mutex_unlock (&manager->initiator_config_mutex);
 
   /* Release the resources */
-  g_free (initiator_name);
-
-  /* Indicate that we handled the method invocation */
-  return TRUE;
-}
-
-static gboolean
-handle_get_initiator_name_raw (UDisksManagerISCSIInitiator *object,
-                               GDBusMethodInvocation       *invocation)
-{
-  UDisksLinuxManagerISCSIInitiator *manager = UDISKS_LINUX_MANAGER_ISCSI_INITIATOR (object);
-  gchar *initiator_name = NULL;
-  GError *error = NULL;
-
-  /* Enter a critical section. */
-  g_mutex_lock (&manager->initiator_config_mutex);
-
-  initiator_name = _get_initiator_name(&error);
-  if (!initiator_name)
-    {
-      g_dbus_method_invocation_take_error (invocation, error);
-      goto out;
-    }
-
-  /* Return the initiator name */
-  udisks_manager_iscsi_initiator_complete_get_initiator_name_raw (object,
-                                                                  invocation,
-                                                                  initiator_name);
-
-out:
-  /* Leave the critical section. */
-  g_mutex_unlock (&manager->initiator_config_mutex);
-
-  /* Release the resources */
-  g_free (initiator_name);
+  g_string_free (content, TRUE);
+  if (initiator_name_fd != -1)
+    close (initiator_name_fd);
 
   /* Indicate that we handled the method invocation */
   return TRUE;
@@ -342,90 +374,82 @@ handle_set_initiator_name (UDisksManagerISCSIInitiator *object,
                            GVariant                    *arg_options)
 {
   UDisksLinuxManagerISCSIInitiator *manager = UDISKS_LINUX_MANAGER_ISCSI_INITIATOR (object);
-  UDisksDaemon *daemon;
-  gchar *contents = NULL;
-  gchar *contents_group;
-  gchar *initiator_name;
-  GKeyFile *key_file;
-  GError *error = NULL;
-
-  daemon = udisks_module_get_daemon (UDISKS_MODULE (manager->module));
+  int initiator_name_fd = -1;
+  GString *content = NULL;
 
   /* Policy check. */
-  if (! udisks_daemon_util_check_authorization_sync (daemon,
-                                                     NULL,
-                                                     ISCSI_MODULE_POLICY_ACTION_ID,
-                                                     arg_options,
-                                                     N_("Authentication is required change iSCSI initiator name"),
-                                                     invocation))
-    return TRUE;
+  UDISKS_DAEMON_CHECK_AUTHORIZATION (manager->daemon,
+                                       NULL,
+                                       iscsi_policy_action_id,
+                                       arg_options,
+                                       N_("Authentication is required change iSCSI initiator name"),
+                                       invocation);
 
   if (!arg_name || strlen (arg_name) == 0)
     {
-      g_dbus_method_invocation_return_error_literal (invocation,
-                                                     UDISKS_ERROR,
-                                                     UDISKS_ERROR_FAILED,
-                                                     N_("Empty initiator name"));
+      g_dbus_method_invocation_return_error (invocation,
+                                             UDISKS_ERROR,
+                                             UDISKS_ERROR_FAILED,
+                                             N_("Empty initiator name"));
       return TRUE;
     }
 
-  /* enter critical section */
+  /* Enter a critical section. */
   g_mutex_lock (&manager->initiator_config_mutex);
 
-  /* first try to read existing file */
-  g_file_get_contents (INITIATOR_FILENAME, &contents, NULL, NULL /* ignore errors */);
+  initiator_name_fd = open (initiator_filename,
+                            O_WRONLY |
+                            O_TRUNC  |
+                            S_IRUSR  |
+                            S_IWUSR  |
+                            S_IRGRP  |
+                            S_IROTH);
 
-  /* key-value pairs have to belong to some group, let's create a fake one just to satisfy the parser needs */
-  contents_group = g_strconcat ("[" FAKE_GROUP_NAME "]\n", contents, NULL);
-  g_free (contents);
-
-  key_file = g_key_file_new ();
-  if (! g_key_file_load_from_data (key_file, contents_group, -1,
-                                   G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS,
-                                   NULL /* ignore errors */))
+  if (initiator_name_fd == -1)
     {
-      /* ignoring errors, leaving the keyfile instance empty (e.g. empty or non-existing config file) */
+      g_dbus_method_invocation_return_error (invocation,
+                                             UDISKS_ERROR,
+                                             UDISKS_ERROR_FAILED,
+                                             N_("Error opening %s while setting iSCSI initiator name: %s"),
+                                             initiator_filename,
+                                             strerror (errno));
+
+      goto mutex_out;
     }
-  g_free (contents_group);
 
-  /* ensure trailing space in the initiator name */
-  if (arg_name[strlen (arg_name) - 1] != ' ')
-    initiator_name = g_strconcat (arg_name, " ", NULL);
-  else
-    initiator_name = g_strdup (arg_name);
-  g_key_file_set_string (key_file, FAKE_GROUP_NAME, INITIATOR_NAME_KEY, initiator_name);
-  g_free (initiator_name);
+  /* Make a new initiator name */
+  content = g_string_new (initiator_name_prefix);
+  g_string_append_printf (content, "%s\n", arg_name);
 
-  contents_group = g_key_file_to_data (key_file, NULL, NULL);
-  /* strip the fake group name */
-  contents = contents_group ? g_strrstr (contents_group, "[" FAKE_GROUP_NAME "]") : NULL;
-  if (contents != NULL)
-    contents += strlen ("[" FAKE_GROUP_NAME "]\n");
-
-  if (contents == NULL)
+  /* Write the new initiator name */
+  if (write (initiator_name_fd, content->str, content->len) != (ssize_t) content->len)
     {
-      g_dbus_method_invocation_return_error_literal (invocation,
-                                                     UDISKS_ERROR,
-                                                     UDISKS_ERROR_FAILED,
-                                                     N_("Error parsing the iSCSI initiator name"));
-    }
-  else if (! g_file_set_contents (INITIATOR_FILENAME, contents, -1, &error))
-    {
-      g_prefix_error (&error, N_("Error writing to %s while setting iSCSI initiator name: "), INITIATOR_FILENAME);
-      g_dbus_method_invocation_take_error (invocation, error);
-    }
-  else
-    {
-      /* finish with no error */
-      udisks_manager_iscsi_initiator_complete_set_initiator_name (object, invocation);
-    }
-  g_free (contents_group);
-  g_key_file_free (key_file);
+      g_dbus_method_invocation_return_error (invocation,
+                                             UDISKS_ERROR,
+                                             UDISKS_ERROR_FAILED,
+                                             N_("Error writing to %s while setting iSCSI initiator name: %s"),
+                                             initiator_filename,
+                                             strerror (errno));
 
-  /* leave critical section */
+      goto mutex_out;
+    }
+
+  /* Finish with no error */
+  udisks_manager_iscsi_initiator_complete_set_initiator_name (object,
+                                                              invocation);
+
+mutex_out:
+  /* Leave the critical section. */
   g_mutex_unlock (&manager->initiator_config_mutex);
 
-  /* indicate that we handled the method invocation */
+out:
+  /* Release the resources */
+  if (content)
+    g_string_free (content, TRUE);
+  if (initiator_name_fd != -1)
+    close (initiator_name_fd);
+
+  /* Indicate that we handled the method invocation */
   return TRUE;
 }
 
@@ -439,22 +463,27 @@ handle_set_initiator_name (UDisksManagerISCSIInitiator *object,
  * Performs firmware discovery (ppc or ibft).
  */
 static gint
+
 discover_firmware (UDisksManagerISCSIInitiator  *object,
                    GVariant                    **nodes,
                    gint                         *nodes_cnt,
                    gchar                       **errorstr)
 {
   UDisksLinuxManagerISCSIInitiator *manager = UDISKS_LINUX_MANAGER_ISCSI_INITIATOR (object);
+  UDisksISCSIState *state = udisks_linux_manager_iscsi_initiator_get_state (manager);
+
+  gint rval;
   struct libiscsi_context *ctx;
   struct libiscsi_node *found_nodes;
-  gint rval;
 
   /* Enter a critical section. */
-  udisks_linux_module_iscsi_lock_libiscsi_context (manager->module);
+  udisks_iscsi_state_lock_libiscsi_context (state);
 
   /* Discovery */
-  ctx = udisks_linux_module_iscsi_get_libiscsi_context (manager->module);
-  rval = libiscsi_discover_firmware (ctx, nodes_cnt, &found_nodes);
+  ctx = udisks_linux_manager_iscsi_initiator_get_iscsi_context (manager);
+  rval = libiscsi_discover_firmware (ctx,
+                                     nodes_cnt,
+                                     &found_nodes);
 
   if (rval == 0)
     *nodes = iscsi_libiscsi_nodes_to_gvariant (found_nodes, *nodes_cnt);
@@ -462,7 +491,7 @@ discover_firmware (UDisksManagerISCSIInitiator  *object,
     *errorstr = g_strdup (libiscsi_get_error_string (ctx));
 
   /* Leave the critical section. */
-  udisks_linux_module_iscsi_unlock_libiscsi_context (manager->module);
+  udisks_iscsi_state_unlock_libiscsi_context (state);
 
   /* Release the resources */
   iscsi_libiscsi_nodes_free (found_nodes);
@@ -478,30 +507,34 @@ handle_discover_send_targets (UDisksManagerISCSIInitiator *object,
                               GVariant                    *arg_options)
 {
   UDisksLinuxManagerISCSIInitiator *manager = UDISKS_LINUX_MANAGER_ISCSI_INITIATOR (object);
-  UDisksDaemon *daemon;
+  UDisksISCSIState *state = udisks_linux_manager_iscsi_initiator_get_state (manager);
   GVariant *nodes = NULL;
   gchar *errorstr = NULL;
   gint err = 0;
   gint nodes_cnt = 0;
 
-  daemon = udisks_module_get_daemon (UDISKS_MODULE (manager->module));
-
   /* Policy check. */
-  UDISKS_DAEMON_CHECK_AUTHORIZATION (daemon,
+  UDISKS_DAEMON_CHECK_AUTHORIZATION (manager->daemon,
                                      NULL,
-                                     ISCSI_MODULE_POLICY_ACTION_ID,
+                                     iscsi_policy_action_id,
                                      arg_options,
                                      N_("Authentication is required to discover targets"),
                                      invocation);
 
   /* Enter a critical section. */
-  udisks_linux_module_iscsi_lock_libiscsi_context (manager->module);
+  udisks_iscsi_state_lock_libiscsi_context (state);
 
   /* Perform the discovery. */
-  err = iscsi_discover_send_targets (manager->module, arg_address, arg_port, arg_options, &nodes, &nodes_cnt, &errorstr);
+  err = iscsi_discover_send_targets (manager->daemon,
+                                     arg_address,
+                                     arg_port,
+                                     arg_options,
+                                     &nodes,
+                                     &nodes_cnt,
+                                     &errorstr);
 
   /* Leave the critical section. */
-  udisks_linux_module_iscsi_unlock_libiscsi_context (manager->module);
+  udisks_iscsi_state_unlock_libiscsi_context (state);
 
   if (err != 0)
     {
@@ -515,10 +548,13 @@ handle_discover_send_targets (UDisksManagerISCSIInitiator *object,
     }
 
   /* Return discovered portals. */
-  udisks_manager_iscsi_initiator_complete_discover_send_targets (object, invocation, nodes, nodes_cnt);
+  udisks_manager_iscsi_initiator_complete_discover_send_targets (object,
+                                                                 invocation,
+                                                                 nodes,
+                                                                 nodes_cnt);
 
 out:
-  g_free (errorstr);
+  g_free ((gpointer) errorstr);
 
   /* Indicate that we handled the method invocation. */
   return TRUE;
@@ -530,24 +566,24 @@ handle_discover_firmware (UDisksManagerISCSIInitiator *object,
                           GVariant                    *arg_options)
 {
   UDisksLinuxManagerISCSIInitiator *manager = UDISKS_LINUX_MANAGER_ISCSI_INITIATOR (object);
-  UDisksDaemon *daemon;
   GVariant *nodes = NULL;
   gint err = 0;
   gint nodes_cnt = 0;
   gchar *errorstr = NULL;
 
-  daemon = udisks_module_get_daemon (UDISKS_MODULE (manager->module));
-
   /* Policy check. */
-  UDISKS_DAEMON_CHECK_AUTHORIZATION (daemon,
+  UDISKS_DAEMON_CHECK_AUTHORIZATION (manager->daemon,
                                      NULL,
-                                     ISCSI_MODULE_POLICY_ACTION_ID,
+                                     iscsi_policy_action_id,
                                      arg_options,
                                      N_("Authentication is required to discover firmware targets"),
                                      invocation);
 
   /* Perform the discovery. */
-  err = discover_firmware (object, &nodes, &nodes_cnt, &errorstr);
+  err = discover_firmware (object,
+                           &nodes,
+                           &nodes_cnt,
+                           &errorstr);
 
   if (err != 0)
     {
@@ -557,12 +593,15 @@ handle_discover_firmware (UDisksManagerISCSIInitiator *object,
                                              iscsi_error_to_udisks_error (err),
                                              N_("Discovery failed: %s"),
                                              errorstr);
-      g_free (errorstr);
+      g_free ((gpointer) errorstr);
       goto out;
     }
 
   /* Return discovered portals. */
-  udisks_manager_iscsi_initiator_complete_discover_firmware (object, invocation, nodes, nodes_cnt);
+  udisks_manager_iscsi_initiator_complete_discover_firmware (object,
+                                                             invocation,
+                                                             nodes,
+                                                             nodes_cnt);
 
 out:
   /* Indicate that we handled the method invocation. */
@@ -580,31 +619,34 @@ handle_login (UDisksManagerISCSIInitiator *object,
               GVariant                    *arg_options)
 {
   UDisksLinuxManagerISCSIInitiator *manager = UDISKS_LINUX_MANAGER_ISCSI_INITIATOR (object);
-  UDisksDaemon *daemon;
+  UDisksISCSIState *state = udisks_linux_manager_iscsi_initiator_get_state (manager);
   gint err = 0;
   gchar *errorstr = NULL;
   GError *error = NULL;
-  UDisksObject *iscsi_object = NULL;
-  UDisksObject *iscsi_session_object = NULL;
-
-  daemon = udisks_module_get_daemon (UDISKS_MODULE (manager->module));
 
   /* Policy check. */
-  UDISKS_DAEMON_CHECK_AUTHORIZATION (daemon,
+  UDISKS_DAEMON_CHECK_AUTHORIZATION (manager->daemon,
                                      NULL,
-                                     ISCSI_MODULE_POLICY_ACTION_ID,
+                                     iscsi_policy_action_id,
                                      arg_options,
                                      N_("Authentication is required to perform iSCSI login"),
                                      invocation);
 
   /* Enter a critical section. */
-  udisks_linux_module_iscsi_lock_libiscsi_context (manager->module);
+  udisks_iscsi_state_lock_libiscsi_context (state);
 
   /* Login */
-  err = iscsi_login (manager->module, arg_name, arg_tpgt, arg_address, arg_port, arg_iface, arg_options, &errorstr);
+  err = iscsi_login (manager->daemon,
+                     arg_name,
+                     arg_tpgt,
+                     arg_address,
+                     arg_port,
+                     arg_iface,
+                     arg_options,
+                     &errorstr);
 
   /* Leave the critical section. */
-  udisks_linux_module_iscsi_unlock_libiscsi_context (manager->module);
+  udisks_iscsi_state_unlock_libiscsi_context (state);
 
   if (err != 0)
     {
@@ -618,13 +660,12 @@ handle_login (UDisksManagerISCSIInitiator *object,
     }
 
   /* sit and wait until the device appears on dbus */
-  iscsi_object = udisks_daemon_wait_for_object_sync (daemon,
-                                                     wait_for_iscsi_object,
-                                                     g_strdup (arg_name),
-                                                     g_free,
-                                                     UDISKS_DEFAULT_WAIT_TIMEOUT,
-                                                     &error);
-   if (iscsi_object == NULL)
+  if (udisks_daemon_wait_for_object_sync (manager->daemon,
+                                          wait_for_iscsi_object,
+                                          g_strdup (arg_name),
+                                          g_free,
+                                          15, /* timeout_seconds */
+                                          &error) == NULL)
     {
       g_prefix_error (&error, "Error waiting for iSCSI device to appear: ");
       g_dbus_method_invocation_take_error (invocation, error);
@@ -633,13 +674,12 @@ handle_login (UDisksManagerISCSIInitiator *object,
 
   if (udisks_manager_iscsi_initiator_get_sessions_supported (UDISKS_MANAGER_ISCSI_INITIATOR (manager)))
     {
-      iscsi_session_object = udisks_daemon_wait_for_object_sync (daemon,
-                                                                 wait_for_iscsi_session_object,
-                                                                 g_strdup (arg_name),
-                                                                 g_free,
-                                                                 UDISKS_DEFAULT_WAIT_TIMEOUT,
-                                                                 &error);
-      if (iscsi_session_object == NULL)
+      if (udisks_daemon_wait_for_object_sync (manager->daemon,
+                                              wait_for_iscsi_session_object,
+                                              g_strdup (arg_name),
+                                              g_free,
+                                              15, /* timeout_seconds */
+                                              &error) == NULL)
         {
           g_prefix_error (&error, "Error waiting for iSCSI session object to appear: ");
           g_dbus_method_invocation_take_error (invocation, error);
@@ -648,12 +688,11 @@ handle_login (UDisksManagerISCSIInitiator *object,
     }
 
   /* Complete DBus call. */
-  udisks_manager_iscsi_initiator_complete_login (object, invocation);
+  udisks_manager_iscsi_initiator_complete_login (object,
+                                                 invocation);
 
 out:
-  g_clear_object (&iscsi_object);
-  g_clear_object (&iscsi_session_object);
-  g_free (errorstr);
+  g_free ((gpointer) errorstr);
 
   /* Indicate that we handled the method invocation. */
   return TRUE;
@@ -670,29 +709,34 @@ handle_logout(UDisksManagerISCSIInitiator *object,
               GVariant                    *arg_options)
 {
   UDisksLinuxManagerISCSIInitiator *manager = UDISKS_LINUX_MANAGER_ISCSI_INITIATOR (object);
-  UDisksDaemon *daemon;
+  UDisksISCSIState *state = udisks_linux_manager_iscsi_initiator_get_state (manager);
   gint err = 0;
   gchar *errorstr = NULL;
   GError *error = NULL;
 
-  daemon = udisks_module_get_daemon (UDISKS_MODULE (manager->module));
-
   /* Policy check. */
-  UDISKS_DAEMON_CHECK_AUTHORIZATION (daemon,
+  UDISKS_DAEMON_CHECK_AUTHORIZATION (manager->daemon,
                                      NULL,
-                                     ISCSI_MODULE_POLICY_ACTION_ID,
+                                     iscsi_policy_action_id,
                                      arg_options,
                                      N_("Authentication is required to perform iSCSI logout"),
                                      invocation);
 
   /* Enter a critical section. */
-  udisks_linux_module_iscsi_lock_libiscsi_context (manager->module);
+  udisks_iscsi_state_lock_libiscsi_context (state);
 
   /* Logout */
-  err = iscsi_logout (manager->module, arg_name, arg_tpgt, arg_address, arg_port, arg_iface, arg_options, &errorstr);
+  err = iscsi_logout (manager->daemon,
+                      arg_name,
+                      arg_tpgt,
+                      arg_address,
+                      arg_port,
+                      arg_iface,
+                      arg_options,
+                      &errorstr);
 
   /* Leave the critical section. */
-  udisks_linux_module_iscsi_unlock_libiscsi_context (manager->module);
+  udisks_iscsi_state_unlock_libiscsi_context (state);
 
   if (err != 0)
     {
@@ -706,11 +750,11 @@ handle_logout(UDisksManagerISCSIInitiator *object,
     }
 
   /* now sit and wait until the device and session disappear on dbus */
-  if (!udisks_daemon_wait_for_object_to_disappear_sync (daemon,
+  if (!udisks_daemon_wait_for_object_to_disappear_sync (manager->daemon,
                                                         wait_for_iscsi_object,
                                                         g_strdup (arg_name),
                                                         g_free,
-                                                        UDISKS_DEFAULT_WAIT_TIMEOUT,
+                                                        15, /* timeout_seconds */
                                                         &error))
     {
       g_prefix_error (&error, "Error waiting for iSCSI device to disappear: ");
@@ -720,11 +764,11 @@ handle_logout(UDisksManagerISCSIInitiator *object,
 
   if (udisks_manager_iscsi_initiator_get_sessions_supported (UDISKS_MANAGER_ISCSI_INITIATOR (manager)))
     {
-      if (!udisks_daemon_wait_for_object_to_disappear_sync (daemon,
+      if (!udisks_daemon_wait_for_object_to_disappear_sync (manager->daemon,
                                                             wait_for_iscsi_session_object,
                                                             g_strdup (arg_name),
                                                             g_free,
-                                                            UDISKS_DEFAULT_WAIT_TIMEOUT,
+                                                            15, /* timeout_seconds */
                                                             &error))
         {
           g_prefix_error (&error, "Error waiting for iSCSI session object to disappear: ");
@@ -734,10 +778,11 @@ handle_logout(UDisksManagerISCSIInitiator *object,
     }
 
   /* Complete DBus call. */
-  udisks_manager_iscsi_initiator_complete_logout (object, invocation);
+  udisks_manager_iscsi_initiator_complete_logout (object,
+                                                  invocation);
 
 out:
-  g_free (errorstr);
+  g_free ((gpointer) errorstr);
 
   /* Indicate that we handled the method invocation. */
   return TRUE;
@@ -750,7 +795,6 @@ udisks_linux_manager_iscsi_initiator_iface_init (UDisksManagerISCSIInitiatorIfac
 {
   iface->handle_get_firmware_initiator_name = handle_get_firmware_initiator_name;
   iface->handle_get_initiator_name = handle_get_initiator_name;
-  iface->handle_get_initiator_name_raw = handle_get_initiator_name_raw;
   iface->handle_set_initiator_name = handle_set_initiator_name;
   iface->handle_discover_send_targets = handle_discover_send_targets;
   iface->handle_discover_firmware = handle_discover_firmware;
