@@ -50,6 +50,10 @@
 #include "udiskslinuxlogicalvolumeobject.h"
 
 #include "udiskslvm2daemonutil.h"
+#include "udiskslvm2dbusutil.h"
+#include "udiskslvm2util.h"
+#include "udisks-lvm2-generated.h"
+
 #include "jobhelpers.h"
 
 /**
@@ -133,33 +137,14 @@ udisks_linux_volume_group_new (void)
 void
 udisks_linux_volume_group_update (UDisksLinuxVolumeGroup *volume_group,
                                   BDLVMVGdata            *vg_info,
-                                  GSList                 *vg_pvs,
                                   gboolean               *needs_polling_ret)
 {
   UDisksVolumeGroup *iface = UDISKS_VOLUME_GROUP (volume_group);
-
   udisks_volume_group_set_name (iface, vg_info->name);
   udisks_volume_group_set_uuid (iface, vg_info->uuid);
   udisks_volume_group_set_size (iface, vg_info->size);
   udisks_volume_group_set_free_size (iface, vg_info->free);
   udisks_volume_group_set_extent_size (iface, vg_info->extent_size);
-
-  {
-    g_autoptr(GStrvBuilder) missing_builder = g_strv_builder_new ();
-    g_auto(GStrv) missing = NULL;
-
-    for (GSList *vg_pvs_p=vg_pvs; vg_pvs_p; vg_pvs_p=vg_pvs_p->next)
-      {
-        BDLVMPVdata *pv_info = vg_pvs_p->data;
-        if (pv_info->missing)
-          g_strv_builder_add (missing_builder, pv_info->pv_uuid);
-      }
-
-    missing = g_strv_builder_end (missing_builder);
-    udisks_volume_group_set_missing_physical_volumes (iface, (const gchar *const *)missing);
-  }
-
-  g_dbus_interface_skeleton_flush (G_DBUS_INTERFACE_SKELETON (iface));
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -190,7 +175,6 @@ handle_poll (UDisksVolumeGroup *_group,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-/*  transfer-full  */
 GList *
 udisks_linux_volume_group_get_logical_volumes (UDisksVolumeGroup *group,
                                                UDisksDaemon      *daemon)
@@ -207,7 +191,7 @@ udisks_linux_volume_group_get_logical_volumes (UDisksVolumeGroup *group,
   objects = udisks_daemon_get_objects (daemon);
   for (l = objects; l != NULL; l = l->next)
     {
-      volume = udisks_object_peek_logical_volume (UDISKS_OBJECT (l->data));
+      volume = udisks_object_peek_logical_volume (UDISKS_OBJECT(l->data));
       if (volume &&
           g_strcmp0 (udisks_logical_volume_get_volume_group (volume),
                      g_dbus_object_get_object_path (object)) == 0)
@@ -263,14 +247,12 @@ handle_delete (UDisksVolumeGroup     *_group,
   GError *error = NULL;
   UDisksLinuxVolumeGroup *group = UDISKS_LINUX_VOLUME_GROUP (_group);
   UDisksLinuxVolumeGroupObject *object = NULL;
-  UDisksLinuxModuleLVM2 *module;
   UDisksDaemon *daemon;
   uid_t caller_uid;
   gboolean teardown_flag = FALSE;
   GList *objects_to_wipe = NULL;
   GList *l;
   VGJobData data;
-  const gchar *device_file = NULL;
 
   g_variant_lookup (arg_options, "tear-down", "b", &teardown_flag);
 
@@ -281,8 +263,7 @@ handle_delete (UDisksVolumeGroup     *_group,
       goto out;
     }
 
-  module = udisks_linux_volume_group_object_get_module (object);
-  daemon = udisks_module_get_daemon (UDISKS_MODULE (module));
+  daemon = udisks_linux_volume_group_object_get_daemon (object);
 
   /* Find physical volumes to wipe. */
   if (arg_wipe)
@@ -314,7 +295,7 @@ handle_delete (UDisksVolumeGroup     *_group,
   /* Policy check. */
   UDISKS_DAEMON_CHECK_AUTHORIZATION (daemon,
                                      UDISKS_OBJECT (object),
-                                     LVM2_POLICY_ACTION_ID,
+                                     lvm2_policy_action_id,
                                      arg_options,
                                      N_("Authentication is required to delete a volume group"),
                                      invocation);
@@ -336,7 +317,6 @@ handle_delete (UDisksVolumeGroup     *_group,
                                                UDISKS_OBJECT (object),
                                                "lvm-vg-delete",
                                                caller_uid,
-                                               FALSE,
                                                vgremove_job_func,
                                                &data,
                                                NULL, /* user_data_free_func */
@@ -348,7 +328,6 @@ handle_delete (UDisksVolumeGroup     *_group,
                                              UDISKS_ERROR_FAILED,
                                              "Error deleting volume group: %s",
                                              error->message);
-      g_clear_error (&error);
       goto out;
     }
 
@@ -356,23 +335,7 @@ handle_delete (UDisksVolumeGroup     *_group,
     {
       UDisksBlock *block = udisks_object_peek_block (l->data);
       if (block)
-        {
-          device_file = udisks_block_get_device (block);
-          if (!bd_lvm_pvremove (device_file, NULL, &error))
-            {
-              udisks_warning ("Failed to wipe PV %s: %s", device_file, error->message);
-              g_clear_error (&error);
-            }
-          else
-            udisks_linux_block_object_trigger_uevent_sync (l->data, UDISKS_DEFAULT_WAIT_TIMEOUT);
-
-          if (bd_lvm_is_tech_avail (BD_LVM_TECH_DEVICES, 0, NULL) &&
-              !bd_lvm_devices_delete (device_file, NULL, NULL, &error))
-            {
-              udisks_warning ("Failed to remove %s from LVM devices file: %s", device_file, error->message);
-              g_clear_error (&error);
-            }
-        }
+        udisks_daemon_util_lvm2_wipe_block (daemon, block, NULL);
     }
 
   udisks_volume_group_complete_delete (_group, invocation);
@@ -385,25 +348,13 @@ handle_delete (UDisksVolumeGroup     *_group,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-typedef struct
-{
-  UDisksLinuxModuleLVM2 *module;
-  const gchar *name;
-} WaitForVolumeGroupObjectData;
-
 static UDisksObject *
 wait_for_volume_group_object (UDisksDaemon *daemon,
                               gpointer      user_data)
 {
-  WaitForVolumeGroupObjectData *data = user_data;
-  UDisksLinuxVolumeGroupObject *object;
+  const gchar *name = user_data;
 
-  object = udisks_linux_module_lvm2_find_volume_group_object (data->module, data->name);
-
-  if (object == NULL)
-    return NULL;
-
-  return g_object_ref (UDISKS_OBJECT (object));
+  return UDISKS_OBJECT (udisks_daemon_util_lvm2_find_volume_group_object (daemon, name));
 }
 
 static gboolean
@@ -415,12 +366,10 @@ handle_rename (UDisksVolumeGroup     *_group,
   GError *error = NULL;
   UDisksLinuxVolumeGroup *group = UDISKS_LINUX_VOLUME_GROUP (_group);
   UDisksLinuxVolumeGroupObject *object = NULL;
-  UDisksLinuxModuleLVM2 *module;
   UDisksDaemon *daemon;
   uid_t caller_uid;
   UDisksObject *group_object = NULL;
   VGJobData data;
-  WaitForVolumeGroupObjectData wait_data;
 
   object = udisks_daemon_util_dup_object (group, &error);
   if (object == NULL)
@@ -429,8 +378,7 @@ handle_rename (UDisksVolumeGroup     *_group,
       goto out;
     }
 
-  module = udisks_linux_volume_group_object_get_module (object);
-  daemon = udisks_module_get_daemon (UDISKS_MODULE (module));
+  daemon = udisks_linux_volume_group_object_get_daemon (object);
 
   if (!udisks_daemon_util_get_caller_uid_sync (daemon,
                                                invocation,
@@ -446,7 +394,7 @@ handle_rename (UDisksVolumeGroup     *_group,
   /* Policy check. */
   UDISKS_DAEMON_CHECK_AUTHORIZATION (daemon,
                                      UDISKS_OBJECT (object),
-                                     LVM2_POLICY_ACTION_ID,
+                                     lvm2_policy_action_id,
                                      options,
                                      N_("Authentication is required to rename a volume group"),
                                      invocation);
@@ -458,7 +406,6 @@ handle_rename (UDisksVolumeGroup     *_group,
                                                UDISKS_OBJECT (object),
                                                "lvm-vg-rename",
                                                caller_uid,
-                                               FALSE,
                                                vgrename_job_func,
                                                &data,
                                                NULL, /* user_data_free_func */
@@ -470,22 +417,19 @@ handle_rename (UDisksVolumeGroup     *_group,
                                              UDISKS_ERROR_FAILED,
                                              "Error renaming volume group: %s",
                                              error->message);
-      g_clear_error (&error);
       goto out;
     }
 
-  wait_data.module = module;
-  wait_data.name = new_name;
   group_object = udisks_daemon_wait_for_object_sync (daemon,
                                                      wait_for_volume_group_object,
-                                                     &wait_data,
+                                                     (gpointer) new_name,
                                                      NULL,
-                                                     UDISKS_DEFAULT_WAIT_TIMEOUT,
+                                                     10, /* timeout_seconds */
                                                      &error);
   if (group_object == NULL)
     {
       g_prefix_error (&error,
-                      "Error waiting for volume group object for '%s': ",
+                      "Error waiting for volume group object for %s",
                       new_name);
       g_dbus_method_invocation_take_error (invocation, error);
       goto out;
@@ -494,7 +438,6 @@ handle_rename (UDisksVolumeGroup     *_group,
   udisks_volume_group_complete_rename (_group,
                                        invocation,
                                        g_dbus_object_get_object_path (G_DBUS_OBJECT (group_object)));
-  g_clear_object (&group_object);
 
  out:
   g_clear_object (&object);
@@ -510,7 +453,6 @@ handle_add_device (UDisksVolumeGroup     *_group,
                    GVariant              *options)
 {
   UDisksLinuxVolumeGroup *group = UDISKS_LINUX_VOLUME_GROUP (_group);
-  UDisksLinuxModuleLVM2 *module;
   UDisksDaemon *daemon;
   UDisksLinuxVolumeGroupObject *object;
   uid_t caller_uid;
@@ -527,8 +469,7 @@ handle_add_device (UDisksVolumeGroup     *_group,
       goto out;
     }
 
-  module = udisks_linux_volume_group_object_get_module (object);
-  daemon = udisks_module_get_daemon (UDISKS_MODULE (module));
+  daemon = udisks_linux_volume_group_object_get_daemon (object);
 
   error = NULL;
   if (!udisks_daemon_util_get_caller_uid_sync (daemon,
@@ -561,7 +502,7 @@ handle_add_device (UDisksVolumeGroup     *_group,
   /* Policy check. */
   UDISKS_DAEMON_CHECK_AUTHORIZATION (daemon,
                                      UDISKS_OBJECT (object),
-                                     LVM2_POLICY_ACTION_ID,
+                                     lvm2_policy_action_id,
                                      options,
                                      N_("Authentication is required to add a device to a volume group"),
                                      invocation);
@@ -587,7 +528,6 @@ handle_add_device (UDisksVolumeGroup     *_group,
                                                    UDISKS_OBJECT (object),
                                                    "lvm-pv-create",
                                                    caller_uid,
-                                                   FALSE,
                                                    pvcreate_job_func,
                                                    &pv_data,
                                                    NULL, /* user_data_free_func */
@@ -600,7 +540,6 @@ handle_add_device (UDisksVolumeGroup     *_group,
                                                  "Error creating LVM metadata on %s: %s",
                                                  pv_data.path,
                                                  error->message);
-          g_clear_error (&error);
           goto out;
         }
     }
@@ -613,7 +552,6 @@ handle_add_device (UDisksVolumeGroup     *_group,
                                                UDISKS_OBJECT (object),
                                                "lvm-vg-add-device",
                                                caller_uid,
-                                               FALSE,
                                                vgextend_job_func,
                                                &data,
                                                NULL, /* user_data_free_func */
@@ -626,7 +564,6 @@ handle_add_device (UDisksVolumeGroup     *_group,
                                              "Error adding %s to volume group: %s",
                                              data.pv_path,
                                              error->message);
-      g_clear_error (&error);
       goto out;
     }
 
@@ -649,7 +586,6 @@ handle_remove_common (UDisksVolumeGroup     *_group,
                       gboolean               arg_wipe)
 {
   UDisksLinuxVolumeGroup *group = UDISKS_LINUX_VOLUME_GROUP (_group);
-  UDisksLinuxModuleLVM2 *module;
   UDisksDaemon *daemon;
   UDisksLinuxVolumeGroupObject *object;
   uid_t caller_uid;
@@ -685,8 +621,7 @@ handle_remove_common (UDisksVolumeGroup     *_group,
       goto out;
     }
 
-  module = udisks_linux_volume_group_object_get_module (object);
-  daemon = udisks_module_get_daemon (UDISKS_MODULE (module));
+  daemon = udisks_linux_volume_group_object_get_daemon (object);
 
   error = NULL;
   if (!udisks_daemon_util_get_caller_uid_sync (daemon,
@@ -719,7 +654,7 @@ handle_remove_common (UDisksVolumeGroup     *_group,
   /* Policy check. */
   UDISKS_DAEMON_CHECK_AUTHORIZATION (daemon,
                                      UDISKS_OBJECT (object),
-                                     LVM2_POLICY_ACTION_ID,
+                                     lvm2_policy_action_id,
                                      options,
                                      authentication_error_msg,
                                      invocation);
@@ -733,7 +668,6 @@ handle_remove_common (UDisksVolumeGroup     *_group,
                                                UDISKS_OBJECT (object),
                                                job_operation,
                                                caller_uid,
-                                               FALSE,
                                                job_func,
                                                &data,
                                                NULL, /* user_data_free_func */
@@ -746,7 +680,6 @@ handle_remove_common (UDisksVolumeGroup     *_group,
                                              (is_remove) ? "Error remove %s from volume group: %s" : "Error emptying %s: %s",
                                              data.pv_path,
                                              error->message);
-      g_clear_error (&error);
       goto out;
     }
 
@@ -756,7 +689,6 @@ handle_remove_common (UDisksVolumeGroup     *_group,
                                                    UDISKS_OBJECT (object),
                                                    "pv-format-erase",
                                                    caller_uid,
-                                                   FALSE,
                                                    pvremove_job_func,
                                                    &data,
                                                    NULL, /* user_data_free_func */
@@ -770,7 +702,6 @@ handle_remove_common (UDisksVolumeGroup     *_group,
                                                  data.pv_path,
                                                  udisks_linux_volume_group_object_get_name (object),
                                                  error->message);
-          g_clear_error (&error);
           goto out;
         }
     }
@@ -810,81 +741,6 @@ handle_empty_device (UDisksVolumeGroup     *_group,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-static gboolean
-handle_remove_missing_physical_volumes (UDisksVolumeGroup     *_group,
-                                        GDBusMethodInvocation *invocation,
-                                        GVariant              *options)
-{
-  UDisksLinuxVolumeGroup *group = UDISKS_LINUX_VOLUME_GROUP (_group);
-  UDisksLinuxModuleLVM2 *module;
-  UDisksDaemon *daemon;
-  UDisksLinuxVolumeGroupObject *object;
-  uid_t caller_uid;
-  GError *error = NULL;
-  VGJobData data;
-
-  object = udisks_daemon_util_dup_object (group, &error);
-  if (object == NULL)
-    {
-      g_dbus_method_invocation_take_error (invocation, error);
-      goto out;
-    }
-
-  module = udisks_linux_volume_group_object_get_module (object);
-  daemon = udisks_module_get_daemon (UDISKS_MODULE (module));
-
-  error = NULL;
-  if (!udisks_daemon_util_get_caller_uid_sync (daemon,
-                                               invocation,
-                                               NULL /* GCancellable */,
-                                               &caller_uid,
-                                               &error))
-    {
-      g_dbus_method_invocation_return_gerror (invocation, error);
-      g_clear_error (&error);
-      goto out;
-    }
-
-  /* Policy check. */
-  UDISKS_DAEMON_CHECK_AUTHORIZATION (daemon,
-                                     UDISKS_OBJECT (object),
-                                     LVM2_POLICY_ACTION_ID,
-                                     options,
-                                     N_("Authentication is required to remove missing physical volumes from a volume group"),
-                                     invocation);
-
-  data.vg_name = udisks_linux_volume_group_object_get_name (object);
-  data.pv_path = NULL;
-
-  if (!udisks_daemon_launch_threaded_job_sync (daemon,
-                                               UDISKS_OBJECT (object),
-                                               "lvm-vg-rem-device",
-                                               caller_uid,
-                                               FALSE,
-                                               vgreduce_job_func,
-                                               &data,
-                                               NULL, /* user_data_free_func */
-                                               NULL, /* GCancellable */
-                                               &error))
-    {
-      g_dbus_method_invocation_return_error (invocation,
-                                             UDISKS_ERROR,
-                                             UDISKS_ERROR_FAILED,
-                                             "Error removing missing physical volumes: %s",
-                                             error->message);
-      g_clear_error (&error);
-      goto out;
-    }
-
-  udisks_volume_group_complete_remove_missing_physical_volumes (_group, invocation);
-
- out:
-  g_clear_object (&object);
-  return TRUE; /* returning TRUE means that we handled the method invocation */
-}
-
-/* ---------------------------------------------------------------------------------------------------- */
-
 struct WaitData {
   UDisksLinuxVolumeGroupObject *group_object;
   const gchar *name;
@@ -895,13 +751,8 @@ wait_for_logical_volume_object (UDisksDaemon *daemon,
                                 gpointer      user_data)
 {
   struct WaitData *data = user_data;
-  UDisksLinuxLogicalVolumeObject *object;
-
-  object = udisks_linux_volume_group_object_find_logical_volume_object (data->group_object, data->name);
-  if (object == NULL)
-    return NULL;
-
-  return g_object_ref (UDISKS_OBJECT (object));
+  return UDISKS_OBJECT (udisks_linux_volume_group_object_find_logical_volume_object (data->group_object,
+                                                                                     data->name));
 }
 
 static const gchar *
@@ -910,33 +761,27 @@ wait_for_logical_volume_path (UDisksLinuxVolumeGroupObject  *group_object,
                               GError                       **error)
 {
   struct WaitData data;
-  UDisksLinuxModuleLVM2 *module;
   UDisksDaemon *daemon;
   UDisksObject *volume_object;
-  const gchar *object_path;
 
   data.group_object = group_object;
   data.name = name;
-  module = udisks_linux_volume_group_object_get_module (group_object);
-  daemon = udisks_module_get_daemon (UDISKS_MODULE (module));
+  daemon = udisks_linux_volume_group_object_get_daemon (group_object);
   volume_object = udisks_daemon_wait_for_object_sync (daemon,
                                                       wait_for_logical_volume_object,
                                                       &data,
                                                       NULL,
-                                                      UDISKS_DEFAULT_WAIT_TIMEOUT,
+                                                      10, /* timeout_seconds */
                                                       error);
   if (volume_object == NULL)
     return NULL;
 
-  object_path = g_dbus_object_get_object_path (G_DBUS_OBJECT (volume_object));
-  g_object_unref (volume_object);
-
-  return object_path;
+  return g_dbus_object_get_object_path (G_DBUS_OBJECT (volume_object));
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-enum VolumeType { VOL_PLAIN, VOL_THIN_POOL, VOL_THIN_VOLUME, VOL_VDO_VOLUME };
+enum VolumeType { VOL_PLAIN, VOL_THIN_POOL, VOL_THIN_VOLUME };
 typedef void (*VolumeCompletionFunc) (UDisksVolumeGroup     *object,
                                       GDBusMethodInvocation *invocation,
                                       const gchar           *result);
@@ -948,24 +793,15 @@ handle_create_volume (UDisksVolumeGroup              *_group,
                       guint64                         arg_size,
                       GVariant                       *options,
                       enum VolumeType                 vol_creation_type,
-                      const gchar                    *arg_pool,
-                      guint64                         arg_virtual_size,
-                      guint64                         arg_index_memory,
-                      gboolean                        arg_compression,
-                      gboolean                        arg_deduplication,
-                      const gchar                    *arg_write_policy,
-                      const gchar                    *arg_layout,
-                      const gchar             *const *arg_pvs)
+                      const gchar                    *arg_pool)
 {
   GError *error = NULL;
   UDisksLinuxVolumeGroup *group = UDISKS_LINUX_VOLUME_GROUP (_group);
   UDisksLinuxVolumeGroupObject *object = NULL;
-  UDisksLinuxModuleLVM2 *module;
   UDisksDaemon *daemon;
   uid_t caller_uid;
   const gchar *lv_objpath;
-  LVJobData data = {0};
-  g_auto(GStrv) pvs = NULL;
+  LVJobData data;
   UDisksLinuxLogicalVolumeObject *pool_object = NULL;
   const gchar *auth_error_msg = NULL;
   UDisksThreadedJobFunc create_function = NULL;
@@ -983,18 +819,12 @@ handle_create_volume (UDisksVolumeGroup              *_group,
       create_function = lvcreate_thin_job_func;
       completion_function = udisks_volume_group_complete_create_thin_volume;
     }
-  else if (VOL_THIN_POOL == vol_creation_type)
+  else
     {
       auth_error_msg = N_("Authentication is required to create a thin pool volume");
       create_function = lvcreate_thin_pool_job_func;
       completion_function = udisks_volume_group_complete_create_thin_pool_volume;
     }
-  else if (VOL_VDO_VOLUME == vol_creation_type)
-    {
-      auth_error_msg = N_("Authentication is required to create a VDO volume");
-      create_function = lvcreate_vdo_job_func;
-      completion_function = udisks_volume_group_complete_create_vdo_volume;
-    }
 
   object = udisks_daemon_util_dup_object (group, &error);
   if (object == NULL)
@@ -1003,8 +833,8 @@ handle_create_volume (UDisksVolumeGroup              *_group,
       goto out;
     }
 
-  module = udisks_linux_volume_group_object_get_module (object);
-  daemon = udisks_module_get_daemon (UDISKS_MODULE (module));
+  daemon = udisks_linux_volume_group_object_get_daemon (object);
+
   if (!udisks_daemon_util_get_caller_uid_sync (daemon,
                                                invocation,
                                                NULL /* GCancellable */,
@@ -1019,7 +849,7 @@ handle_create_volume (UDisksVolumeGroup              *_group,
   /* Policy check. */
   UDISKS_DAEMON_CHECK_AUTHORIZATION (daemon,
                                      UDISKS_OBJECT (object),
-                                     LVM2_POLICY_ACTION_ID,
+                                     lvm2_policy_action_id,
                                      options,
                                      auth_error_msg,
                                      invocation);
@@ -1028,54 +858,6 @@ handle_create_volume (UDisksVolumeGroup              *_group,
   data.new_lv_name = arg_name;
   data.new_lv_size = arg_size;
 
-  if (VOL_PLAIN == vol_creation_type)
-    {
-      if (arg_layout != NULL
-          && g_strcmp0 (arg_layout, "linear") != 0
-          && !g_str_has_prefix (arg_layout, "raid")
-          && g_strcmp0 (arg_layout, "mirror") != 0)
-        {
-          g_dbus_method_invocation_return_error (invocation, UDISKS_ERROR, UDISKS_ERROR_FAILED,
-                                                 "Not a valid layout for a plain volume: %s", arg_layout);
-          goto out;
-        }
-
-      data.new_lv_layout = arg_layout;
-      data.new_lv_stripes = 0;
-      data.new_lv_mirrors = 0;
-
-      if (arg_pvs)
-        {
-          gint32 n_subvolumes = g_strv_length ((gchar **)arg_pvs);
-          g_variant_lookup (options, "subvolumes", "i", &n_subvolumes);
-
-          if (g_strcmp0 (arg_layout, "raid0") == 0)
-            data.new_lv_stripes = n_subvolumes;
-          else if (g_strcmp0 (arg_layout, "raid1") == 0)
-            data.new_lv_mirrors = n_subvolumes - 1;
-          else if (g_strcmp0 (arg_layout, "raid10") == 0) {
-            if (n_subvolumes % 2 != 0)
-              {
-                g_dbus_method_invocation_return_error (invocation, UDISKS_ERROR, UDISKS_ERROR_FAILED,
-                                                       "Number of subvolumes for raid10 must be even");
-                goto out;
-              }
-            data.new_lv_stripes = n_subvolumes / 2;
-          } else if (g_strcmp0 (arg_layout, "raid4") == 0 ||g_strcmp0 (arg_layout, "raid5") == 0)
-            data.new_lv_stripes = n_subvolumes - 1;
-          else if (g_strcmp0 (arg_layout, "raid6") == 0)
-            data.new_lv_stripes = n_subvolumes - 2;
-
-          pvs = udisks_daemon_util_lvm2_gather_pvs (daemon, object, arg_pvs, &error);
-          if (pvs == NULL)
-            {
-              g_dbus_method_invocation_take_error (invocation, error);
-              goto out;
-            }
-          data.new_lv_pvs = (const gchar **)pvs;
-        }
-    }
-
   if (VOL_THIN_POOL == vol_creation_type)
     data.extent_size = udisks_volume_group_get_extent_size (UDISKS_VOLUME_GROUP (group));
 
@@ -1091,21 +873,10 @@ handle_create_volume (UDisksVolumeGroup              *_group,
       data.pool_name = udisks_linux_logical_volume_object_get_name (pool_object);
     }
 
-  if (VOL_VDO_VOLUME == vol_creation_type)
-    {
-      data.pool_name = arg_pool;
-      data.virtual_size = arg_virtual_size;
-      data.index_memory = arg_index_memory;
-      data.compression = arg_compression;
-      data.deduplication = arg_deduplication;
-      data.write_policy = arg_write_policy;
-    }
-
   if (!udisks_daemon_launch_threaded_job_sync (daemon,
                                                UDISKS_OBJECT (object),
                                                "lvm-vg-create-volume",
                                                caller_uid,
-                                               FALSE,
                                                create_function,
                                                &data,
                                                NULL, /* user_data_free_func */
@@ -1117,7 +888,6 @@ handle_create_volume (UDisksVolumeGroup              *_group,
                                              UDISKS_ERROR_FAILED,
                                              "Error creating volume: %s",
                                              error->message);
-      g_clear_error (&error);
       goto out;
     }
 
@@ -1125,7 +895,7 @@ handle_create_volume (UDisksVolumeGroup              *_group,
   if (lv_objpath == NULL)
     {
       g_prefix_error (&error,
-                      "Error waiting for logical volume object for '%s': ",
+                      "Error waiting for logical volume object for %s",
                       arg_name);
       g_dbus_method_invocation_take_error (invocation, error);
       goto out;
@@ -1147,22 +917,7 @@ handle_create_plain_volume (UDisksVolumeGroup     *_group,
                             GVariant              *options)
 {
   return handle_create_volume(_group, invocation, arg_name, arg_size, options,
-                              VOL_PLAIN, NULL, 0, 0, FALSE, FALSE, NULL,
-                              NULL, NULL);
-}
-
-static gboolean
-handle_create_plain_volume_with_layout (UDisksVolumeGroup     *_group,
-                                        GDBusMethodInvocation *invocation,
-                                        const gchar           *arg_name,
-                                        guint64                arg_size,
-                                        const gchar           *arg_layout,
-                                        const gchar *const    *arg_pvs,
-                                        GVariant              *options)
-{
-  return handle_create_volume(_group, invocation, arg_name, arg_size, options,
-                              VOL_PLAIN, NULL, 0, 0, FALSE, FALSE, NULL,
-                              arg_layout, arg_pvs);
+                              VOL_PLAIN, NULL);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -1175,8 +930,7 @@ handle_create_thin_pool_volume (UDisksVolumeGroup     *_group,
                                 GVariant              *options)
 {
   return handle_create_volume(_group, invocation, arg_name, arg_size, options,
-                              VOL_THIN_POOL, NULL, 0, 0, FALSE, FALSE, NULL,
-                              NULL, NULL);
+                              VOL_THIN_POOL, NULL);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -1190,27 +944,7 @@ handle_create_thin_volume (UDisksVolumeGroup     *_group,
                            GVariant              *options)
 {
   return handle_create_volume(_group, invocation, arg_name, arg_size, options,
-                              VOL_THIN_VOLUME, arg_pool, 0, 0, FALSE, FALSE, NULL,
-                              NULL, NULL);
-}
-
-static gboolean
-handle_create_vdo_volume (UDisksVolumeGroup     *_group,
-                          GDBusMethodInvocation *invocation,
-                          const gchar           *arg_name,
-                          const gchar           *arg_pool,
-                          guint64                arg_size,
-                          guint64                arg_virtual_size,
-                          guint64                arg_index_memory,
-                          gboolean               arg_compression,
-                          gboolean               arg_deduplication,
-                          const gchar           *arg_write_policy,
-                          GVariant              *options)
-{
-  return handle_create_volume (_group, invocation, arg_name, arg_size, options,
-                               VOL_VDO_VOLUME, arg_pool, arg_virtual_size, arg_index_memory,
-                               arg_compression, arg_deduplication, arg_write_policy,
-                               NULL, NULL);
+                              VOL_THIN_VOLUME, arg_pool);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -1226,12 +960,8 @@ volume_group_iface_init (UDisksVolumeGroupIface *iface)
   iface->handle_add_device = handle_add_device;
   iface->handle_remove_device = handle_remove_device;
   iface->handle_empty_device = handle_empty_device;
-  iface->handle_remove_missing_physical_volumes = handle_remove_missing_physical_volumes;
 
   iface->handle_create_plain_volume = handle_create_plain_volume;
-  iface->handle_create_plain_volume_with_layout = handle_create_plain_volume_with_layout;
   iface->handle_create_thin_pool_volume = handle_create_thin_pool_volume;
   iface->handle_create_thin_volume = handle_create_thin_volume;
-
-  iface->handle_create_vdo_volume = handle_create_vdo_volume;
 }
