@@ -35,7 +35,6 @@
 #include <sys/ioctl.h>
 #include <linux/fs.h>
 
-#include <blockdev/fs.h>
 #include <blockdev/lvm.h>
 
 #include <limits.h>
@@ -50,6 +49,9 @@
 #include <src/udisksmodulemanager.h>
 
 #include "udiskslvm2daemonutil.h"
+#include "udiskslvm2dbusutil.h"
+#include "udiskslvm2state.h"
+#include "udisks-lvm2-generated.h"
 
 /**
  * SECTION:udiskslvm2daemonutil
@@ -80,6 +82,63 @@ udisks_daemon_util_lvm2_block_is_unused (UDisksBlock  *block,
   return TRUE;
 }
 
+static gboolean
+run_sync (const gchar *prog, ...)
+{
+  va_list ap;
+  GError **error;
+  enum { max_argc = 20 };
+  const gchar *argv[max_argc+1];
+  int argc = 0;
+  const gchar *arg;
+  gchar *standard_output;
+  gchar *standard_error;
+  gint exit_status;
+
+  argv[argc++] = prog;
+  va_start (ap, prog);
+  while ((arg = va_arg (ap, const gchar *)))
+    {
+      if (argc < max_argc)
+        argv[argc] = arg;
+      argc++;
+    }
+  error = va_arg (ap, GError **);
+  va_end (ap);
+
+  if (argc > max_argc)
+    {
+      g_set_error (error, UDISKS_ERROR, UDISKS_ERROR_FAILED,
+                   "Too many arguments.");
+      return FALSE;
+    }
+
+  argv[argc] = NULL;
+  if (!g_spawn_sync (NULL,
+                     (gchar **)argv,
+                     NULL,
+                     G_SPAWN_SEARCH_PATH,
+                     NULL,
+                     NULL,
+                     &standard_output,
+                     &standard_error,
+                     &exit_status,
+                     error))
+    return FALSE;
+
+  if (!g_spawn_check_exit_status (exit_status, error))
+    {
+      g_prefix_error (error, "stdout: '%s', stderr: '%s', ", standard_output, standard_error);
+      g_free (standard_output);
+      g_free (standard_error);
+      return FALSE;
+    }
+
+  g_free (standard_output);
+  g_free (standard_error);
+  return TRUE;
+}
+
 gboolean
 udisks_daemon_util_lvm2_wipe_block (UDisksDaemon  *daemon,
                                     UDisksBlock   *block,
@@ -88,23 +147,22 @@ udisks_daemon_util_lvm2_wipe_block (UDisksDaemon  *daemon,
   UDisksObject *block_object;
   UDisksPhysicalVolume *physical_volume;
   const gchar *volume_group_objpath;
-  UDisksObject *volume_group_object = NULL;
+  UDisksObject *volume_group_object;
   UDisksVolumeGroup *volume_group;
   gchar *volume_group_name = NULL;
   gboolean was_partitioned;
+
   const gchar *device_file;
+  int fd = -1;
+  gchar zeroes[512];
   gboolean ret = TRUE;
   GError *local_error = NULL;
 
   /* Find the name of the volume group that this device is a physical
    * member of, if any.  Easy.
    */
-  block_object = udisks_daemon_util_dup_object (block, error);
-  if (block_object == NULL)
-    {
-      ret = FALSE;
-      goto out;
-    }
+
+  block_object = UDISKS_OBJECT (g_dbus_interface_get_object (G_DBUS_INTERFACE (block)));
   physical_volume = udisks_object_peek_physical_volume (block_object);
   if (physical_volume)
     {
@@ -122,26 +180,48 @@ udisks_daemon_util_lvm2_wipe_block (UDisksDaemon  *daemon,
 
   device_file = udisks_block_get_device (block);
 
-  if (!bd_fs_clean (device_file, FALSE, &local_error))
+  /* Remove partition table */
+  memset (zeroes, 0, 512);
+  fd = open (device_file, O_RDWR | O_EXCL);
+  if (fd < 0)
     {
       g_set_error (error, UDISKS_ERROR, UDISKS_ERROR_FAILED,
-                   "%s", local_error->message);
-      g_clear_error (&local_error);
+                   "Error opening device %s for wiping: %m",
+                   device_file);
       ret = FALSE;
       goto out;
     }
 
-  if (was_partitioned &&
-      !udisks_linux_block_object_reread_partition_table (UDISKS_LINUX_BLOCK_OBJECT (block_object), error))
+  if (write (fd, zeroes, 512) != 512)
+    {
+      g_set_error (error, UDISKS_ERROR, UDISKS_ERROR_FAILED,
+                   "Error erasing device %s: %m",
+                   device_file);
+      ret = FALSE;
+      goto out;
+    }
+
+  if (was_partitioned && ioctl (fd, BLKRRPART, NULL) < 0)
+    {
+      g_set_error (error, UDISKS_ERROR, UDISKS_ERROR_FAILED,
+                   "Error removing partition devices of %s: %m",
+                   device_file);
+      ret = FALSE;
+      goto out;
+    }
+  close (fd);
+  fd = -1;
+
+  /* wipe other labels */
+  if (!run_sync ("wipefs", "-a", device_file, NULL, error))
     {
       ret = FALSE;
       goto out;
     }
 
   /* Try to bring affected volume group back into consistency. */
-  if (volume_group_name != NULL &&
-      !bd_lvm_vgreduce (volume_group_name, NULL /* device */, NULL /* extra */, &local_error))
-    {
+  if (volume_group_name != NULL)
+    if (!bd_lvm_vgreduce (volume_group_name, NULL /* device */, NULL /* extra */, &local_error)) {
       udisks_warning ("%s", local_error->message);
       g_clear_error (&local_error);
     }
@@ -153,21 +233,39 @@ udisks_daemon_util_lvm2_wipe_block (UDisksDaemon  *daemon,
    *
    * https://bugzilla.redhat.com/show_bug.cgi?id=1063813
    */
-  if (!bd_lvm_pvscan (device_file, TRUE, NULL, &local_error))
+  if (!run_sync ("pvscan", "--cache", device_file, NULL, &local_error))
     {
       udisks_warning ("%s", local_error->message);
       g_clear_error (&local_error);
     }
 
  out:
-  g_clear_object (&volume_group_object);
-  g_clear_object (&block_object);
+  if (fd >= 0)
+    close (fd);
   g_free (volume_group_name);
   return ret;
 }
 
 /* -------------------------------------------------------------------------------- */
 
+UDisksLinuxVolumeGroupObject *
+udisks_daemon_util_lvm2_find_volume_group_object (UDisksDaemon *daemon,
+                                                  const gchar  *name)
+{
+  UDisksLVM2State *state;
+  UDisksModuleManager *manager;
+
+  manager = udisks_daemon_get_module_manager (daemon);
+  g_assert (manager != NULL);
+
+  state = (UDisksLVM2State *) udisks_module_manager_get_module_state_pointer (manager, LVM2_MODULE_NAME);
+  g_assert (state != NULL);
+
+  return g_hash_table_lookup (udisks_lvm2_state_get_name_to_volume_group (state), name);
+}
+
+/* -------------------------------------------------------------------------------- */
+
 gboolean
 udisks_daemon_util_lvm2_name_is_reserved (const gchar *name)
 {
@@ -193,74 +291,3 @@ udisks_daemon_util_lvm2_trigger_udev (const gchar *device_file)
   if (fd >= 0)
     close (fd);
 }
-
-/* -------------------------------------------------------------------------------- */
-
-GStrv
-udisks_daemon_util_lvm2_gather_pvs (UDisksDaemon                  *daemon,
-                                    UDisksLinuxVolumeGroupObject  *vgroup_object,
-                                    const gchar *const            *arg_pvs,
-                                    GError                       **error)
-{
-  GStrv result = g_new0 (gchar *, g_strv_length ((GStrv)arg_pvs)+1);
-
-  for (int i = 0; arg_pvs[i] != NULL; i++)
-    {
-      UDisksObject *pvol_object = NULL;
-      UDisksBlock *block = NULL;
-      UDisksPhysicalVolume *pvol = NULL;
-
-      pvol_object = udisks_daemon_find_object (daemon, arg_pvs[i]);
-      if (pvol_object == NULL)
-        {
-          g_set_error (error,
-                       UDISKS_ERROR,
-                       UDISKS_ERROR_FAILED,
-                       "Invalid object path %s at index %u",
-                       arg_pvs[i], i);
-          goto out;
-        }
-
-      block = udisks_object_get_block (pvol_object);
-      pvol = udisks_object_get_physical_volume (pvol_object);
-      if (block == NULL || pvol == NULL)
-        {
-          g_set_error (error,
-                       UDISKS_ERROR,
-                       UDISKS_ERROR_FAILED,
-                       "Object path %s for index %u is not a physical volume",
-                       arg_pvs[i], i);
-          if (block)
-            g_object_unref (block);
-          if (pvol)
-            g_object_unref (pvol);
-          g_object_unref (pvol_object);
-          goto out;
-        }
-
-      if (g_strcmp0 (udisks_physical_volume_get_volume_group (pvol),
-                     g_dbus_object_get_object_path (G_DBUS_OBJECT (vgroup_object))) != 0)
-        {
-          g_set_error (error,
-                       UDISKS_ERROR,
-                       UDISKS_ERROR_FAILED,
-                       "Physical volume %s for index %u does not belong to this volume group",
-                       arg_pvs[i], i);
-          g_object_unref (pvol_object);
-          g_object_unref (pvol);
-          g_object_unref (block);
-          goto out;
-        }
-
-      result[i] = udisks_block_dup_device (block);
-      g_object_unref (block);
-      g_object_unref (pvol);
-      g_object_unref (pvol_object);
-    }
-
-  return result;
-
- out:
-  g_strfreev (result);
-  return NULL;
-}
