@@ -27,6 +27,8 @@
 #include <string.h>
 #include <stdlib.h>
 
+#include <blockdev/crypto.h>
+
 #include <glib/gstdio.h>
 
 #include "udiskslogging.h"
@@ -43,6 +45,9 @@
 #include "udiskscrypttabentry.h"
 #include "udiskscrypttabmonitor.h"
 #include "udisksspawnedjob.h"
+#include "udiskssimplejob.h"
+
+#define MAX_TCRYPT_KEYFILES 256
 
 /**
  * SECTION:udiskslinuxencrypted
@@ -103,34 +108,6 @@ udisks_linux_encrypted_new (void)
   return UDISKS_ENCRYPTED (g_object_new (UDISKS_TYPE_LINUX_ENCRYPTED,
                                          NULL));
 }
-/* ---------------------------------------------------------------------------------------------------- */
-
-static void
-update_child_configuration (UDisksLinuxEncrypted   *encrypted,
-                            UDisksLinuxBlockObject *object)
-{
-  UDisksDaemon *daemon = udisks_linux_block_object_get_daemon (object);
-  UDisksBlock *block = udisks_object_peek_block (UDISKS_OBJECT (object));
-
-  udisks_encrypted_set_child_configuration
-    (UDISKS_ENCRYPTED (encrypted),
-     udisks_linux_find_child_configuration (daemon,
-                                            udisks_block_get_id_uuid (block)));
-}
-
-/**
- * udisks_linux_encrypted_update:
- * @encrypted: A #UDisksLinuxEncrypted.
- * @object: The enclosing #UDisksLinuxBlockObject instance.
- *
- * Updates the interface.
- */
-void
-udisks_linux_encrypted_update (UDisksLinuxEncrypted   *encrypted,
-                               UDisksLinuxBlockObject *object)
-{
-  update_child_configuration (encrypted, object);
-}
 
 /* ---------------------------------------------------------------------------------------------------- */
 
@@ -168,12 +145,106 @@ wait_for_cleartext_object (UDisksDaemon *daemon,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+static void
+update_child_configuration (UDisksLinuxEncrypted   *encrypted,
+                            UDisksLinuxBlockObject *object)
+{
+  UDisksDaemon *daemon = udisks_linux_block_object_get_daemon (object);
+  UDisksBlock *block = udisks_object_peek_block (UDISKS_OBJECT (object));
+
+  udisks_encrypted_set_child_configuration
+    (UDISKS_ENCRYPTED (encrypted),
+     udisks_linux_find_child_configuration (daemon,
+                                            udisks_block_get_id_uuid (block)));
+}
+
+static void
+update_metadata_size (UDisksLinuxEncrypted   *encrypted,
+                      UDisksLinuxBlockObject *object)
+{
+  UDisksLinuxDevice *device;
+  guint64 metadata_size;
+  GError *error = NULL;
+
+  device = udisks_linux_block_object_get_device (object);
+
+  metadata_size = bd_crypto_luks_get_metadata_size (g_udev_device_get_device_file (device->udev_device),
+                                                    &error);
+
+  if (error != NULL)
+  {
+    udisks_warning ("Error getting '%s' metadata_size: %s (%s, %d)",
+                    g_udev_device_get_device_file (device->udev_device),
+                    error->message,
+                    g_quark_to_string (error->domain),
+                    error->code);
+    g_clear_error (&error);
+  }
+
+  g_object_unref (device);
+  udisks_encrypted_set_metadata_size (UDISKS_ENCRYPTED (encrypted), metadata_size);
+}
+
+static void
+update_cleartext_device (UDisksLinuxEncrypted   *encrypted,
+                         UDisksLinuxBlockObject *object)
+{
+  UDisksObject *cleartext_object = NULL;
+  UDisksDaemon *daemon = udisks_linux_block_object_get_daemon (object);
+  const gchar *encrypted_path = g_dbus_object_get_object_path (G_DBUS_OBJECT (object));
+
+  /* wait_for_cleartext is used primarly in unlock but does exactly what we
+     want -- returns a cleartext object for an encrypted object */
+  cleartext_object = wait_for_cleartext_object (daemon, (gpointer) encrypted_path);
+
+  if (cleartext_object) {
+      udisks_encrypted_set_cleartext_device (UDISKS_ENCRYPTED (encrypted),
+                                             g_dbus_object_get_object_path (G_DBUS_OBJECT (cleartext_object)));
+  }
+  else {
+    udisks_encrypted_set_cleartext_device (UDISKS_ENCRYPTED (encrypted), "/");
+  }
+}
+
+/**
+ * udisks_linux_encrypted_update:
+ * @encrypted: A #UDisksLinuxEncrypted.
+ * @object: The enclosing #UDisksLinuxBlockObject instance.
+ *
+ * Updates the interface.
+ */
+void
+udisks_linux_encrypted_update (UDisksLinuxEncrypted   *encrypted,
+                               UDisksLinuxBlockObject *object)
+{
+  UDisksBlock *block = udisks_object_peek_block (UDISKS_OBJECT (object));
+
+  udisks_linux_block_encrypted_lock (block);
+
+  update_child_configuration (encrypted, object);
+  update_cleartext_device (encrypted, object);
+
+  /* set block type according to hint_encryption_type */
+  if (udisks_linux_block_is_unknown_crypto (block))
+  {
+    if (g_strcmp0(udisks_encrypted_get_hint_encryption_type(UDISKS_ENCRYPTED(encrypted)), "TCRYPT") == 0)
+      udisks_block_set_id_type (block, "crypto_TCRYPT");
+  }
+
+  update_metadata_size (encrypted, object);
+
+  udisks_linux_block_encrypted_unlock (block);
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 static gboolean
 check_crypttab (UDisksBlock  *block,
                 gboolean      load_passphrase,
                 gboolean     *out_found,
                 gchar       **out_name,
                 gchar       **out_passphrase,
+                gsize        *out_passphrase_len,
                 gchar       **out_options,
                 GError      **error)
 {
@@ -200,7 +271,7 @@ check_crypttab (UDisksBlock  *block,
                 {
                   if (!g_file_get_contents (passphrase_path,
                                             out_passphrase,
-                                            NULL,
+                                            out_passphrase_len,
                                             error))
                     {
                       g_variant_unref (details);
@@ -270,11 +341,21 @@ handle_unlock (UDisksEncrypted        *encrypted,
   gboolean is_in_crypttab = FALSE;
   gchar *crypttab_name = NULL;
   gchar *crypttab_passphrase = NULL;
+  gsize crypttab_passphrase_len = 0;
   gchar *crypttab_options = NULL;
   gchar *device = NULL;
+  gchar *old_hint_encryption_type;
   gboolean read_only = FALSE;
+  gboolean is_hidden = FALSE;
+  gboolean is_system = FALSE;
+  guint32 pim = 0;
   GString *effective_passphrase = NULL;
-  LuksJobData data;
+  GVariant *keyfiles_variant = NULL;
+  const gchar *keyfiles[MAX_TCRYPT_KEYFILES] = {};
+  CryptoJobData data;
+  gboolean is_luks;
+  gboolean handle_as_tcrypt;
+  void *open_func;
 
   object = udisks_daemon_util_dup_object (encrypted, &error);
   if (object == NULL)
@@ -286,20 +367,45 @@ handle_unlock (UDisksEncrypted        *encrypted,
   block = udisks_object_peek_block (object);
   daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
   state = udisks_daemon_get_state (daemon);
+  is_luks = udisks_linux_block_is_luks (block);
+  handle_as_tcrypt = udisks_linux_block_is_tcrypt (block) || udisks_linux_block_is_unknown_crypto (block);
+
+  /* get TCRYPT options */
+  if (handle_as_tcrypt)
+    {
+      g_variant_lookup (options, "hidden", "b", &is_hidden);
+      g_variant_lookup (options, "system", "b", &is_system);
+      g_variant_lookup (options, "pim", "u", &pim);
+
+      /* get keyfiles */
+      keyfiles_variant = g_variant_lookup_value(options, "keyfiles", G_VARIANT_TYPE_ARRAY);
+      if (keyfiles_variant)
+        {
+          GVariantIter iter;
+          const gchar *path;
+          uint i = 0;
+
+          g_variant_iter_init (&iter, keyfiles_variant);
+          while (g_variant_iter_next (&iter, "&s", &path) && i < MAX_TCRYPT_KEYFILES)
+            {
+              keyfiles[i] = path;
+              i++;
+            }
+        }
+    }
 
   /* TODO: check if the device is mentioned in /etc/crypttab (see crypttab(5)) - if so use that
    *
    *       Of course cryptsetup(8) don't support that, see https://bugzilla.redhat.com/show_bug.cgi?id=692258
    */
 
-  /* Fail if the device is not a LUKS device */
-  if (!(g_strcmp0 (udisks_block_get_id_usage (block), "crypto") == 0 &&
-        g_strcmp0 (udisks_block_get_id_type (block), "crypto_LUKS") == 0))
+  /* Fail if the device is not a LUKS or possible TCRYPT device */
+  if (!(is_luks || handle_as_tcrypt))
     {
       g_dbus_method_invocation_return_error (invocation,
                                              UDISKS_ERROR,
                                              UDISKS_ERROR_FAILED,
-                                             "Device %s does not appear to be a LUKS device",
+                                             "Device %s does not appear to be a LUKS or TCRYPT device",
                                              udisks_block_get_device (block));
       goto out;
     }
@@ -324,51 +430,50 @@ handle_unlock (UDisksEncrypted        *encrypted,
       goto out;
     }
 
-  /* we need the uid of the caller for the unlocked-luks file */
+  /* we need the uid of the caller for the unlocked-crypto-dev file */
   error = NULL;
-  if (!udisks_daemon_util_get_caller_uid_sync (daemon, invocation, NULL /* GCancellable */, &caller_uid, NULL, NULL, &error))
+  if (!udisks_daemon_util_get_caller_uid_sync (daemon, invocation, NULL /* GCancellable */, &caller_uid, &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
       g_clear_error (&error);
       goto out;
     }
 
-  /* fallback mechanism: keyfile_contents -> passphrase -> crypttab_passphrase -> error (no key) */
-  if (!udisks_variant_lookup_binary (options, "keyfile_contents", &effective_passphrase)) {
-    if (passphrase && (strlen (passphrase) > 0))
-      {
-        effective_passphrase = g_string_new (passphrase);
-      }
-    else
-      {
-        /* check if in crypttab file */
-        error = NULL;
-        if (!check_crypttab (block,
-                             TRUE,
-                             &is_in_crypttab,
-                             &crypttab_name,
-                             &crypttab_passphrase,
-                             &crypttab_options,
-                             &error))
-          {
-            g_dbus_method_invocation_take_error (invocation, error);
-            goto out;
-          }
-        if (is_in_crypttab && crypttab_passphrase != NULL && strlen (crypttab_passphrase) > 0)
-          {
-            effective_passphrase = g_string_new (crypttab_passphrase);
-          }
-        else
-          {
-            g_dbus_method_invocation_return_error (invocation,
-                                                   UDISKS_ERROR,
-                                                   UDISKS_ERROR_FAILED,
-                                                   "No key available to unlock device %s",
-                                                   udisks_block_get_device (block));
-            goto out;
-          }
-      }
-  }
+  /* check if in crypttab file */
+  error = NULL;
+  if (!check_crypttab (block,
+                       TRUE,
+                       &is_in_crypttab,
+                       &crypttab_name,
+                       &crypttab_passphrase,
+                       &crypttab_passphrase_len,
+                       &crypttab_options,
+                       &error))
+    {
+      g_dbus_method_invocation_take_error (invocation, error);
+      goto out;
+    }
+
+  /* fallback mechanism: keyfile_contents (for LUKS) -> passphrase -> crypttab_passphrase -> TCRYPT keyfiles -> error (no key) */
+  if (is_luks && udisks_variant_lookup_binary (options, "keyfile_contents", &effective_passphrase))
+    {
+      /* effective_passphrase was set to keyfile_contents, nothing more to do here */
+    }
+  else if (passphrase && (strlen (passphrase) > 0))
+    effective_passphrase = g_string_new (passphrase);
+  else if (is_in_crypttab && crypttab_passphrase != NULL && crypttab_passphrase_len > 0)
+    effective_passphrase = g_string_new_len (crypttab_passphrase, crypttab_passphrase_len);
+  else if (keyfiles[0] != NULL)
+    effective_passphrase = g_string_new (NULL);
+  else
+    {
+      g_dbus_method_invocation_return_error (invocation,
+                                             UDISKS_ERROR,
+                                             UDISKS_ERROR_FAILED,
+                                             "No key available to unlock device %s",
+                                             udisks_block_get_device (block));
+      goto out;
+    }
 
   /* Now, check that the user is actually authorized to unlock the device.
    */
@@ -407,8 +512,24 @@ handle_unlock (UDisksEncrypted        *encrypted,
   /* calculate the name to use */
   if (is_in_crypttab && crypttab_name != NULL)
     name = g_strdup (crypttab_name);
+  else {
+    if (is_luks)
+      name = g_strdup_printf ("luks-%s", udisks_block_get_id_uuid (block));
+    else
+      /* TCRYPT devices don't have a UUID, so we use the device number instead */
+      name = g_strdup_printf ("tcrypt-%" G_GUINT64_FORMAT, udisks_block_get_device_number (block));
+  }
+
+  /* save old encryption type to be able to restore it */
+  old_hint_encryption_type = udisks_encrypted_dup_hint_encryption_type (encrypted);
+
+  /* Set hint_encryption type. We have to do this before the
+   * actual unlock, in order to have this set before the device
+   * update triggered by the unlock. */
+  if (is_luks)
+    udisks_encrypted_set_hint_encryption_type (encrypted, "LUKS");
   else
-    name = g_strdup_printf ("luks-%s", udisks_block_get_id_uuid (block));
+    udisks_encrypted_set_hint_encryption_type (encrypted, "TCRYPT");
 
   device = udisks_block_dup_device (block);
 
@@ -419,13 +540,23 @@ handle_unlock (UDisksEncrypted        *encrypted,
   data.device = device;
   data.map_name = name;
   data.passphrase = effective_passphrase;
+  data.keyfiles = keyfiles;
+  data.pim = pim;
+  data.hidden = is_hidden;
+  data.system = is_system;
   data.read_only = read_only;
 
+  if (is_luks)
+    open_func = luks_open_job_func;
+  else
+    open_func = tcrypt_open_job_func;
+
+  udisks_linux_block_encrypted_lock (block);
   if (!udisks_daemon_launch_threaded_job_sync (daemon,
                                                object,
                                                "encrypted-unlock",
                                                caller_uid,
-                                               luks_open_job_func,
+                                               open_func,
                                                &data,
                                                NULL, /* user_data_free_func */
                                                NULL, /* cancellable */
@@ -437,8 +568,22 @@ handle_unlock (UDisksEncrypted        *encrypted,
                                              "Error unlocking %s: %s",
                                              udisks_block_get_device (block),
                                              error->message);
+
+      /* Restore the old encryption type if the unlock failed, because
+       * in this case we don't know for sure if we used the correct
+       * encryption type. */
+      udisks_encrypted_set_hint_encryption_type (encrypted, old_hint_encryption_type);
+      udisks_linux_block_encrypted_unlock (block);
       goto out;
     }
+  else
+    {
+      /* We have to free old_hint_encryption_type if and only if it was not
+       * used in udisks_encrypted_set_hint_encryption_type() */
+      g_free (old_hint_encryption_type);
+    }
+
+  udisks_linux_block_encrypted_unlock (block);
 
   /* Determine the resulting cleartext object */
   error = NULL;
@@ -446,7 +591,7 @@ handle_unlock (UDisksEncrypted        *encrypted,
                                                          wait_for_cleartext_object,
                                                          g_strdup (g_dbus_object_get_object_path (G_DBUS_OBJECT (object))),
                                                          g_free,
-                                                         10, /* timeout_seconds */
+                                                         20, /* timeout_seconds */
                                                          &error);
   if (cleartext_object == NULL)
     {
@@ -458,18 +603,18 @@ handle_unlock (UDisksEncrypted        *encrypted,
     }
   cleartext_block = udisks_object_peek_block (cleartext_object);
 
-  udisks_notice ("Unlocked LUKS device %s as %s",
+  udisks_notice ("Unlocked device %s as %s",
                  udisks_block_get_device (block),
                  udisks_block_get_device (cleartext_block));
 
   cleartext_device = udisks_linux_block_object_get_device (UDISKS_LINUX_BLOCK_OBJECT (cleartext_object));
 
-  /* update the unlocked-luks file */
-  udisks_state_add_unlocked_luks (state,
-                                  udisks_block_get_device_number (cleartext_block),
-                                  udisks_block_get_device_number (block),
-                                  g_udev_device_get_sysfs_attr (cleartext_device->udev_device, "dm/uuid"),
-                                  caller_uid);
+  /* update the unlocked-crypto-dev file */
+  udisks_state_add_unlocked_crypto_dev (state,
+                                        udisks_block_get_device_number (cleartext_block),
+                                        udisks_block_get_device_number (block),
+                                        g_udev_device_get_sysfs_attr (cleartext_device->udev_device, "dm/uuid"),
+                                        caller_uid);
 
   udisks_encrypted_complete_unlock (encrypted,
                                     invocation,
@@ -508,8 +653,12 @@ udisks_linux_encrypted_lock (UDisksLinuxEncrypted   *encrypted,
   dev_t cleartext_device_from_file;
   uid_t caller_uid;
   gboolean ret;
-  LuksJobData data;
+  CryptoJobData data;
   GError *loc_error = NULL;
+  gchar *cleartext_path = NULL;
+  void *close_func;
+  gboolean is_luks;
+  gboolean handle_as_tcrypt;
 
   object = udisks_daemon_util_dup_object (encrypted, error);
   if (object == NULL)
@@ -521,20 +670,21 @@ udisks_linux_encrypted_lock (UDisksLinuxEncrypted   *encrypted,
   block = udisks_object_peek_block (object);
   daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
   state = udisks_daemon_get_state (daemon);
+  is_luks = udisks_linux_block_is_luks (block);
+  handle_as_tcrypt = udisks_linux_block_is_tcrypt (block) || udisks_linux_block_is_unknown_crypto (block);
 
   /* TODO: check if the device is mentioned in /etc/crypttab (see crypttab(5)) - if so use that
    *
    *       Of course cryptsetup(8) don't support that, see https://bugzilla.redhat.com/show_bug.cgi?id=692258
    */
 
-  /* Fail if the device is not a LUKS device */
-  if (!(g_strcmp0 (udisks_block_get_id_usage (block), "crypto") == 0 &&
-        g_strcmp0 (udisks_block_get_id_type (block), "crypto_LUKS") == 0))
+  /* Fail if the device is not a LUKS or possible TCRYPT device */
+  if (!(is_luks || handle_as_tcrypt))
     {
       g_set_error (error,
                    UDISKS_ERROR,
                    UDISKS_ERROR_FAILED,
-                   "Device %s does not appear to be a LUKS device",
+                   "Device %s does not appear to be a LUKS or TCRYPT device",
                    udisks_block_get_device (block));
       ret = FALSE;
       goto out;
@@ -559,12 +709,12 @@ udisks_linux_encrypted_lock (UDisksLinuxEncrypted   *encrypted,
     }
   cleartext_block = udisks_object_peek_block (cleartext_object);
 
-  cleartext_device_from_file = udisks_state_find_unlocked_luks (state,
-                                                                udisks_block_get_device_number (block),
-                                                                &unlocked_by_uid);
+  cleartext_device_from_file = udisks_state_find_unlocked_crypto_dev (state,
+                                                                      udisks_block_get_device_number (block),
+                                                                      &unlocked_by_uid);
   if (cleartext_device_from_file == 0)
     {
-      /* allow locking stuff not mentioned in unlocked-luks, but treat it like root unlocked it */
+      /* allow locking stuff not mentioned in unlocked-crypto-dev, but treat it like root unlocked it */
       unlocked_by_uid = 0;
     }
 
@@ -573,8 +723,6 @@ udisks_linux_encrypted_lock (UDisksLinuxEncrypted   *encrypted,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
-                                               NULL,
-                                               NULL,
                                                error))
     {
       ret = FALSE;
@@ -609,11 +757,17 @@ udisks_linux_encrypted_lock (UDisksLinuxEncrypted   *encrypted,
   device = udisks_linux_block_object_get_device (UDISKS_LINUX_BLOCK_OBJECT (cleartext_object));
   data.map_name = g_udev_device_get_sysfs_attr (device->udev_device, "dm/name");
 
+  if (is_luks)
+    close_func = luks_close_job_func;
+  else
+    close_func = tcrypt_close_job_func;
+
+  udisks_linux_block_encrypted_lock (block);
   if (!udisks_daemon_launch_threaded_job_sync (daemon,
                                                object,
                                                "encrypted-lock",
                                                caller_uid,
-                                               luks_close_job_func,
+                                               close_func,
                                                &data,
                                                NULL, /* user_data_free_func */
                                                NULL, /* cancellable */
@@ -628,10 +782,31 @@ udisks_linux_encrypted_lock (UDisksLinuxEncrypted   *encrypted,
                    loc_error->message);
       ret = FALSE;
       g_clear_error (&loc_error);
+      udisks_linux_block_encrypted_unlock (block);
       goto out;
     }
 
-  udisks_notice ("Locked LUKS device %s (was unlocked as %s)",
+  udisks_linux_block_encrypted_unlock (block);
+
+  cleartext_path = g_strdup (g_dbus_object_get_object_path (G_DBUS_OBJECT (object)));
+  if (! udisks_daemon_wait_for_object_to_disappear_sync (daemon,
+                                                         wait_for_cleartext_object,
+                                                         cleartext_path,
+                                                         NULL,
+                                                         10,
+                                                         &loc_error))
+    {
+      g_set_error (error,
+                   UDISKS_ERROR,
+                   UDISKS_ERROR_FAILED,
+                   "Error waiting for cleartext object to disappear after locking the device: %s",
+                   loc_error->message);
+      g_clear_error (&loc_error);
+      ret = FALSE;
+      goto out;
+    }
+
+  udisks_notice ("Locked device %s (was unlocked as %s)",
                  udisks_block_get_device (block),
                  udisks_block_get_device (cleartext_block));
   ret = TRUE;
@@ -642,6 +817,7 @@ udisks_linux_encrypted_lock (UDisksLinuxEncrypted   *encrypted,
   if (cleartext_object != NULL)
     g_object_unref (cleartext_object);
   g_clear_object (&object);
+  g_free (cleartext_path);
 
   return ret;
 }
@@ -679,7 +855,7 @@ handle_change_passphrase (UDisksEncrypted        *encrypted,
   const gchar *action_id;
   GError *error = NULL;
   gchar *device = NULL;
-  LuksJobData data;
+  CryptoJobData data = { NULL, NULL, NULL, NULL, NULL, 0, 0, FALSE, FALSE, FALSE, NULL };
 
   object = udisks_daemon_util_dup_object (encrypted, &error);
   if (object == NULL)
@@ -696,9 +872,8 @@ handle_change_passphrase (UDisksEncrypted        *encrypted,
    *       Of course cryptsetup(8) don't support that, see https://bugzilla.redhat.com/show_bug.cgi?id=692258
    */
 
-  /* Fail if the device is not a LUKS device */
-  if (!(g_strcmp0 (udisks_block_get_id_usage (block), "crypto") == 0 &&
-        g_strcmp0 (udisks_block_get_id_type (block), "crypto_LUKS") == 0))
+  /* Fail if the device is not a LUKS device (changing passphrase is currently not supported for TCRYPT devices) */
+  if (!udisks_linux_block_is_luks (block))
     {
       g_dbus_method_invocation_return_error (invocation,
                                              UDISKS_ERROR,
@@ -709,7 +884,7 @@ handle_change_passphrase (UDisksEncrypted        *encrypted,
     }
 
   error = NULL;
-  if (!udisks_daemon_util_get_caller_uid_sync (daemon, invocation, NULL /* GCancellable */, &caller_uid, NULL, NULL, &error))
+  if (!udisks_daemon_util_get_caller_uid_sync (daemon, invocation, NULL /* GCancellable */, &caller_uid, &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
       g_clear_error (&error);
@@ -749,6 +924,7 @@ handle_change_passphrase (UDisksEncrypted        *encrypted,
                                      &(data.new_passphrase)))
     data.new_passphrase = g_string_new (new_passphrase);
 
+  udisks_linux_block_encrypted_lock (block);
   if (!udisks_daemon_launch_threaded_job_sync (daemon,
                                                object,
                                                "encrypted-modify",
@@ -766,9 +942,12 @@ handle_change_passphrase (UDisksEncrypted        *encrypted,
                                              udisks_block_get_device (block),
                                              error->message);
       g_clear_error (&error);
+      udisks_linux_block_encrypted_unlock (block);
       goto out;
     }
 
+  udisks_linux_block_encrypted_unlock (block);
+
   udisks_encrypted_complete_change_passphrase (encrypted, invocation);
 
  out:
@@ -782,10 +961,156 @@ handle_change_passphrase (UDisksEncrypted        *encrypted,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+/* runs in thread dedicated to handling method call */
+static gboolean
+handle_resize (UDisksEncrypted       *encrypted,
+               GDBusMethodInvocation *invocation,
+               guint64                size,
+               GVariant              *options)
+{
+  UDisksObject *object = NULL;
+  UDisksBlock *block;
+  UDisksObject *cleartext_object = NULL;
+  UDisksBlock *cleartext_block;
+  UDisksDaemon *daemon;
+  uid_t caller_uid;
+  const gchar *action_id = NULL;
+  const gchar *message = NULL;
+  GError *error = NULL;
+  UDisksBaseJob *job = NULL;
+  GString *effective_passphrase = NULL;
+
+  object = udisks_daemon_util_dup_object (encrypted, &error);
+  if (object == NULL)
+    {
+      g_dbus_method_invocation_take_error (invocation, error);
+      goto out;
+    }
+
+  block = udisks_object_peek_block (object);
+  daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
+
+  /* Fail if the device is not a LUKS device */
+  if (!(g_strcmp0 (udisks_block_get_id_usage (block), "crypto") == 0 &&
+        g_strcmp0 (udisks_block_get_id_type (block), "crypto_LUKS") == 0))
+    {
+      g_dbus_method_invocation_return_error (invocation,
+                                             UDISKS_ERROR,
+                                             UDISKS_ERROR_FAILED,
+                                             "Device %s does not appear to be a LUKS device",
+                                             udisks_block_get_device (block));
+      goto out;
+    }
+
+  error = NULL;
+  if (!udisks_daemon_util_get_caller_uid_sync (daemon, invocation, NULL /* GCancellable */, &caller_uid, &error))
+    {
+      g_dbus_method_invocation_return_gerror (invocation, error);
+      g_clear_error (&error);
+      goto out;
+    }
+
+  /* Fail if device is not unlocked */
+  cleartext_object = udisks_daemon_wait_for_object_sync (daemon,
+                                                         wait_for_cleartext_object,
+                                                         g_strdup (g_dbus_object_get_object_path (G_DBUS_OBJECT (object))),
+                                                         g_free,
+                                                         0, /* timeout_seconds */
+                                                         NULL); /* error */
+  if (cleartext_object == NULL)
+    {
+      g_dbus_method_invocation_return_error (invocation,
+                                             UDISKS_ERROR,
+                                             UDISKS_ERROR_FAILED,
+                                             "Device %s is not unlocked",
+                                             udisks_block_get_device (block));
+      goto out;
+    }
+  cleartext_block = udisks_object_peek_block (cleartext_object);
+
+  action_id = "org.freedesktop.udisks2.modify-device";
+  /* Translators: Shown in authentication dialog when the user
+   * requests resizing a encrypted block device.
+   *
+   * Do not translate $(drive), it's a placeholder and
+   * will be replaced by the name of the drive/device in question
+   */
+  message = N_("Authentication is required to resize the encrypted device $(drive)");
+  if (! udisks_daemon_util_setup_by_user (daemon, object, caller_uid))
+    {
+      if (udisks_block_get_hint_system (block))
+        {
+          action_id = "org.freedesktop.udisks2.modify-device-system";
+        }
+      else if (! udisks_daemon_util_on_user_seat (daemon, UDISKS_OBJECT (object), caller_uid))
+        {
+          action_id = "org.freedesktop.udisks2.modify-device-other-seat";
+        }
+    }
+
+  /* Check that the user is actually authorized to resize the device. */
+  if (! udisks_daemon_util_check_authorization_sync (daemon,
+                                                     object,
+                                                     action_id,
+                                                     options,
+                                                     message,
+                                                     invocation))
+    goto out;
+
+  job = udisks_daemon_launch_simple_job (daemon,
+                                         UDISKS_OBJECT (object),
+                                         "encrypted-resize",
+                                         caller_uid,
+                                         NULL);
+  if (job == NULL)
+    {
+      g_dbus_method_invocation_return_error (invocation, UDISKS_ERROR, UDISKS_ERROR_FAILED,
+                                             "Failed to create a job object");
+      goto out;
+    }
+
+  if (udisks_variant_lookup_binary (options, "keyfile_contents", &effective_passphrase))
+    ;
+  else if (udisks_variant_lookup_binary (options, "passphrase", &effective_passphrase))
+    ;
+  else
+    effective_passphrase = NULL;
+
+  /* TODO: implement progress parsing for udisks_job_set_progress(_valid) */
+  if (! bd_crypto_luks_resize_luks2_blob (udisks_block_get_device (cleartext_block),
+                                          size / 512,
+                                          effective_passphrase ? (const guint8*) effective_passphrase->str : NULL,
+                                          effective_passphrase ? effective_passphrase->len : 0,
+                                          &error))
+    {
+      g_dbus_method_invocation_return_error (invocation,
+                                             UDISKS_ERROR,
+                                             UDISKS_ERROR_FAILED,
+                                             "Error resizing encrypted device %s: %s",
+                                             udisks_block_get_device (cleartext_block),
+                                             error->message);
+      udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), FALSE, error->message);
+      goto out;
+    }
+
+  udisks_encrypted_complete_resize (encrypted, invocation);
+  udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), TRUE, NULL);
+
+ out:
+  g_clear_object (&cleartext_object);
+  g_clear_object (&object);
+  g_clear_error (&error);
+  udisks_string_wipe_and_free (effective_passphrase);
+  return TRUE; /* returning TRUE means that we handled the method invocation */
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 static void
 encrypted_iface_init (UDisksEncryptedIface *iface)
 {
   iface->handle_unlock              = handle_unlock;
   iface->handle_lock                = handle_lock;
   iface->handle_change_passphrase   = handle_change_passphrase;
+  iface->handle_resize              = handle_resize;
 }
