@@ -323,7 +323,7 @@ handle_loop_setup (UDisksManager          *object,
 
   /* we need the uid of the caller for the loop file */
   error = NULL;
-  if (!udisks_daemon_util_get_caller_uid_sync (manager->daemon, invocation, NULL /* GCancellable */, &caller_uid, NULL, NULL, &error))
+  if (!udisks_daemon_util_get_caller_uid_sync (manager->daemon, invocation, NULL /* GCancellable */, &caller_uid, &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
       g_clear_error (&error);
@@ -401,6 +401,15 @@ handle_loop_setup (UDisksManager          *object,
 
   loop_device = g_strdup_printf ("/dev/%s", loop_name);
 
+  /* Update the udisks loop state file (/run/udisks2/loop) with information
+   * about the new loop device created by us.
+   */
+  udisks_state_add_loop (udisks_daemon_get_state (manager->daemon),
+                        loop_device,
+                        path,
+                        fd_statbuf_valid ? fd_statbuf.st_dev : 0,
+                        caller_uid);
+
   /* Determine the resulting object */
   error = NULL;
   wait_data.loop_device = loop_device;
@@ -424,16 +433,6 @@ handle_loop_setup (UDisksManager          *object,
                  loop_device,
                  path);
 
-  /* Update the udisks loop state file (/run/udisks2/loop) with information
-     about the new loop device created by us. We need to manually trigger
-     uevent for the device after this to update Loop interface properties. */
-  udisks_state_add_loop (udisks_daemon_get_state (manager->daemon),
-                        loop_device,
-                        path,
-                        fd_statbuf_valid ? fd_statbuf.st_dev : 0,
-                        caller_uid);
-  udisks_linux_block_object_trigger_uevent (UDISKS_LINUX_BLOCK_OBJECT (loop_object));
-
   udisks_manager_complete_loop_setup (object,
                                       invocation,
                                       NULL, /* fd_list */
@@ -523,7 +522,6 @@ handle_mdraid_create (UDisksManager         *_object,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
-                                               NULL, NULL,
                                                &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -660,7 +658,7 @@ handle_mdraid_create (UDisksManager         *_object,
           g_dbus_method_invocation_return_error (invocation,
                                                  UDISKS_ERROR,
                                                  UDISKS_ERROR_FAILED,
-                                                 "Error opening device %s: %m",
+                                                 "Error opening device %s while creating mdraid: %m",
                                                  device_file);
           g_free (device_file);
           success = FALSE;
@@ -839,6 +837,7 @@ handle_mdraid_create (UDisksManager         *_object,
       udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), success, NULL);
     }
 
+  g_strfreev ((gchar **) disks);
   g_free (raid_device_file);
   g_free (raid_node);
   g_free (array_name);
@@ -1020,7 +1019,8 @@ get_block_objects (UDisksManager *manager,
 {
   UDisksLinuxManager *linux_manager = UDISKS_LINUX_MANAGER (manager);
   GDBusObjectManagerServer *object_manager = NULL;
-  GList *objects, *objects_p = NULL;
+  GList *objects = NULL;
+  GList *objects_p = NULL;
   GSList *ret = NULL;
 
   object_manager = udisks_daemon_get_object_manager (linux_manager->daemon);
@@ -1049,7 +1049,8 @@ handle_get_block_devices (UDisksManager         *object,
                           GDBusMethodInvocation *invocation,
                           GVariant              *arg_options)
 {
-  GSList *blocks, *blocks_p = NULL;
+  GSList *blocks = NULL;
+  GSList *blocks_p = NULL;
   const gchar **block_paths = NULL;
   guint num_blocks = 0;
   guint i = 0;
@@ -1069,6 +1070,90 @@ handle_get_block_devices (UDisksManager         *object,
   return TRUE;  /* returning TRUE means that we handled the method invocation */
 }
 
+static gboolean
+compare_paths (UDisksManager         *object,
+               UDisksBlock           *block,
+               gchar                 *path)
+{
+  const gchar *const *symlinks = NULL;
+
+  if (g_strcmp0 (udisks_block_get_device (block), path) == 0)
+    return TRUE;
+
+  symlinks = udisks_block_get_symlinks (block);
+  if (symlinks != NULL)
+    {
+      for (guint i = 0; symlinks[i] != NULL; i++)
+        if (g_strcmp0 (symlinks[i], path) == 0)
+          return TRUE;
+    }
+
+  return FALSE;
+}
+
+static gboolean
+handle_resolve_device (UDisksManager         *object,
+                       GDBusMethodInvocation *invocation,
+                       GVariant              *arg_devspec,
+                       GVariant              *arg_options)
+{
+
+  gchar *devpath = NULL;
+  gchar *devuuid = NULL;
+  gchar *devlabel = NULL;
+
+  GSList *blocks = NULL;
+  GSList *blocks_p = NULL;
+  guint num_blocks = 0;
+
+  GSList *ret = NULL;
+  GSList *ret_p = NULL;
+  guint num_found = 0;
+  const gchar **ret_paths = NULL;
+
+  gboolean found = FALSE;
+  guint i = 0;
+
+  g_variant_lookup (arg_devspec, "path", "s", &devpath);
+  g_variant_lookup (arg_devspec, "uuid", "s", &devuuid);
+  g_variant_lookup (arg_devspec, "label", "s", &devlabel);
+
+  blocks = get_block_objects (object, &num_blocks);
+
+  for (blocks_p = blocks; blocks_p != NULL; blocks_p = blocks_p->next)
+    {
+      if (devpath != NULL)
+          found = compare_paths (object, blocks_p->data, devpath);
+
+      if (devuuid != NULL)
+          found = g_strcmp0 (udisks_block_get_id_uuid (blocks_p->data), devuuid) == 0;
+      if (devlabel != NULL)
+          found = g_strcmp0 (udisks_block_get_id_label (blocks_p->data), devlabel) == 0;
+
+      if (found)
+        {
+          ret = g_slist_prepend (ret, g_object_ref (blocks_p->data));
+          num_found++;
+        }
+    }
+
+    ret_paths = g_new0 (const gchar *, num_found + 1);
+    for (i = 0,ret_p = ret; ret_p != NULL; ret_p = ret_p->next, i++)
+      {
+        ret_paths[i] = g_dbus_object_get_object_path (g_dbus_interface_get_object (G_DBUS_INTERFACE (ret_p->data)));
+      }
+
+    udisks_manager_complete_resolve_device (object,
+                                            invocation,
+                                            ret_paths);
+
+    g_slist_free_full (blocks, g_object_unref);
+    g_slist_free_full (ret, g_object_unref);
+
+    return TRUE;  /* returning TRUE means that we handled the method invocation */
+
+}
+
 /* ---------------------------------------------------------------------------------------------------- */
 
 static void
@@ -1082,4 +1167,5 @@ manager_iface_init (UDisksManagerIface *iface)
   iface->handle_can_check = handle_can_check;
   iface->handle_can_repair = handle_can_repair;
   iface->handle_get_block_devices = handle_get_block_devices;
+  iface->handle_resolve_device = handle_resolve_device;
 }
